import ares_struct_pb2 as _ares_struct_pb2
from scripting import completion_pb2 as _completion_pb2
from scripting import diagnostic_pb2 as _diagnostic_pb2
from scripting import navigation_pb2 as _navigation_pb2
from scripting import script_summary_step_pb2 as _script_summary_step_pb2
from scripting import script_symbol_metadata_pb2 as _script_symbol_metadata_pb2
from scripting import signature_help_pb2 as _signature_help_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ScriptExecutionRequest(_message.Message):
    __slots__ = ("script",)
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    script: str
    def __init__(self, script: _Optional[str] = ...) -> None: ...

class ScriptExecutionEvent(_message.Message):
    __slots__ = ("sequence", "execution_started", "execution_completed", "execution_failed", "console_output", "function_started", "function_completed", "function_failed")
    SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_STARTED_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_COMPLETED_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_FAILED_FIELD_NUMBER: _ClassVar[int]
    CONSOLE_OUTPUT_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_STARTED_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_COMPLETED_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_FAILED_FIELD_NUMBER: _ClassVar[int]
    sequence: int
    execution_started: ExecutionStarted
    execution_completed: ExecutionCompleted
    execution_failed: ExecutionFailed
    console_output: ConsoleOutput
    function_started: FunctionStarted
    function_completed: FunctionCompleted
    function_failed: FunctionFailed
    def __init__(self, sequence: _Optional[int] = ..., execution_started: _Optional[_Union[ExecutionStarted, _Mapping]] = ..., execution_completed: _Optional[_Union[ExecutionCompleted, _Mapping]] = ..., execution_failed: _Optional[_Union[ExecutionFailed, _Mapping]] = ..., console_output: _Optional[_Union[ConsoleOutput, _Mapping]] = ..., function_started: _Optional[_Union[FunctionStarted, _Mapping]] = ..., function_completed: _Optional[_Union[FunctionCompleted, _Mapping]] = ..., function_failed: _Optional[_Union[FunctionFailed, _Mapping]] = ...) -> None: ...

class ExecutionStarted(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ExecutionCompleted(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ExecutionFailed(_message.Message):
    __slots__ = ("error",)
    ERROR_FIELD_NUMBER: _ClassVar[int]
    error: str
    def __init__(self, error: _Optional[str] = ...) -> None: ...

class ConsoleOutput(_message.Message):
    __slots__ = ("output",)
    OUTPUT_FIELD_NUMBER: _ClassVar[int]
    output: str
    def __init__(self, output: _Optional[str] = ...) -> None: ...

class FunctionStarted(_message.Message):
    __slots__ = ("call_id", "parent_call_id", "invocation")
    CALL_ID_FIELD_NUMBER: _ClassVar[int]
    PARENT_CALL_ID_FIELD_NUMBER: _ClassVar[int]
    INVOCATION_FIELD_NUMBER: _ClassVar[int]
    call_id: str
    parent_call_id: str
    invocation: _script_summary_step_pb2.ScriptFunctionInvocation
    def __init__(self, call_id: _Optional[str] = ..., parent_call_id: _Optional[str] = ..., invocation: _Optional[_Union[_script_summary_step_pb2.ScriptFunctionInvocation, _Mapping]] = ...) -> None: ...

class FunctionCompleted(_message.Message):
    __slots__ = ("call_id", "result")
    CALL_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    call_id: str
    result: _ares_struct_pb2.AresValue
    def __init__(self, call_id: _Optional[str] = ..., result: _Optional[_Union[_ares_struct_pb2.AresValue, _Mapping]] = ...) -> None: ...

class FunctionFailed(_message.Message):
    __slots__ = ("call_id", "error")
    CALL_ID_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    call_id: str
    error: str
    def __init__(self, call_id: _Optional[str] = ..., error: _Optional[str] = ...) -> None: ...

class ValidateScriptRequest(_message.Message):
    __slots__ = ("script",)
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    script: str
    def __init__(self, script: _Optional[str] = ...) -> None: ...

class ValidateScriptResponse(_message.Message):
    __slots__ = ("diagnostics",)
    DIAGNOSTICS_FIELD_NUMBER: _ClassVar[int]
    diagnostics: _containers.RepeatedCompositeFieldContainer[_diagnostic_pb2.Diagnostic]
    def __init__(self, diagnostics: _Optional[_Iterable[_Union[_diagnostic_pb2.Diagnostic, _Mapping]]] = ...) -> None: ...

class ScriptSummaryRequest(_message.Message):
    __slots__ = ("script", "include_user_functions", "include_lambdas")
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_USER_FUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_LAMBDAS_FIELD_NUMBER: _ClassVar[int]
    script: str
    include_user_functions: bool
    include_lambdas: bool
    def __init__(self, script: _Optional[str] = ..., include_user_functions: bool = ..., include_lambdas: bool = ...) -> None: ...

class ScriptSummaryResponse(_message.Message):
    __slots__ = ("steps", "diagnostics")
    STEPS_FIELD_NUMBER: _ClassVar[int]
    DIAGNOSTICS_FIELD_NUMBER: _ClassVar[int]
    steps: _containers.RepeatedCompositeFieldContainer[_script_summary_step_pb2.ScriptFunctionInvocation]
    diagnostics: _containers.RepeatedCompositeFieldContainer[_diagnostic_pb2.Diagnostic]
    def __init__(self, steps: _Optional[_Iterable[_Union[_script_summary_step_pb2.ScriptFunctionInvocation, _Mapping]]] = ..., diagnostics: _Optional[_Iterable[_Union[_diagnostic_pb2.Diagnostic, _Mapping]]] = ...) -> None: ...

class SymbolMetadataRequest(_message.Message):
    __slots__ = ("script", "cursor_line", "cursor_column", "identifier")
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    CURSOR_LINE_FIELD_NUMBER: _ClassVar[int]
    CURSOR_COLUMN_FIELD_NUMBER: _ClassVar[int]
    IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    script: str
    cursor_line: int
    cursor_column: int
    identifier: str
    def __init__(self, script: _Optional[str] = ..., cursor_line: _Optional[int] = ..., cursor_column: _Optional[int] = ..., identifier: _Optional[str] = ...) -> None: ...

class SymbolMetadataResponse(_message.Message):
    __slots__ = ("found", "metadata", "diagnostics")
    FOUND_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    DIAGNOSTICS_FIELD_NUMBER: _ClassVar[int]
    found: bool
    metadata: _script_symbol_metadata_pb2.ScriptSymbolMetadata
    diagnostics: _containers.RepeatedCompositeFieldContainer[_diagnostic_pb2.Diagnostic]
    def __init__(self, found: bool = ..., metadata: _Optional[_Union[_script_symbol_metadata_pb2.ScriptSymbolMetadata, _Mapping]] = ..., diagnostics: _Optional[_Iterable[_Union[_diagnostic_pb2.Diagnostic, _Mapping]]] = ...) -> None: ...

class ReferencesRequest(_message.Message):
    __slots__ = ("script", "cursor_line", "cursor_column", "identifier", "include_definition")
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    CURSOR_LINE_FIELD_NUMBER: _ClassVar[int]
    CURSOR_COLUMN_FIELD_NUMBER: _ClassVar[int]
    IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_DEFINITION_FIELD_NUMBER: _ClassVar[int]
    script: str
    cursor_line: int
    cursor_column: int
    identifier: str
    include_definition: bool
    def __init__(self, script: _Optional[str] = ..., cursor_line: _Optional[int] = ..., cursor_column: _Optional[int] = ..., identifier: _Optional[str] = ..., include_definition: bool = ...) -> None: ...

class ReferencesResponse(_message.Message):
    __slots__ = ("locations", "diagnostics")
    LOCATIONS_FIELD_NUMBER: _ClassVar[int]
    DIAGNOSTICS_FIELD_NUMBER: _ClassVar[int]
    locations: _containers.RepeatedCompositeFieldContainer[_navigation_pb2.SymbolLocation]
    diagnostics: _containers.RepeatedCompositeFieldContainer[_diagnostic_pb2.Diagnostic]
    def __init__(self, locations: _Optional[_Iterable[_Union[_navigation_pb2.SymbolLocation, _Mapping]]] = ..., diagnostics: _Optional[_Iterable[_Union[_diagnostic_pb2.Diagnostic, _Mapping]]] = ...) -> None: ...

class SignatureHelpRequest(_message.Message):
    __slots__ = ("script", "cursor_line", "cursor_column")
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    CURSOR_LINE_FIELD_NUMBER: _ClassVar[int]
    CURSOR_COLUMN_FIELD_NUMBER: _ClassVar[int]
    script: str
    cursor_line: int
    cursor_column: int
    def __init__(self, script: _Optional[str] = ..., cursor_line: _Optional[int] = ..., cursor_column: _Optional[int] = ...) -> None: ...

class SignatureHelpResponse(_message.Message):
    __slots__ = ("signature_help", "diagnostics")
    SIGNATURE_HELP_FIELD_NUMBER: _ClassVar[int]
    DIAGNOSTICS_FIELD_NUMBER: _ClassVar[int]
    signature_help: _signature_help_pb2.SignatureHelp
    diagnostics: _containers.RepeatedCompositeFieldContainer[_diagnostic_pb2.Diagnostic]
    def __init__(self, signature_help: _Optional[_Union[_signature_help_pb2.SignatureHelp, _Mapping]] = ..., diagnostics: _Optional[_Iterable[_Union[_diagnostic_pb2.Diagnostic, _Mapping]]] = ...) -> None: ...

class CompletionRequest(_message.Message):
    __slots__ = ("script", "cursor_line", "cursor_column", "trigger_character")
    SCRIPT_FIELD_NUMBER: _ClassVar[int]
    CURSOR_LINE_FIELD_NUMBER: _ClassVar[int]
    CURSOR_COLUMN_FIELD_NUMBER: _ClassVar[int]
    TRIGGER_CHARACTER_FIELD_NUMBER: _ClassVar[int]
    script: str
    cursor_line: int
    cursor_column: int
    trigger_character: str
    def __init__(self, script: _Optional[str] = ..., cursor_line: _Optional[int] = ..., cursor_column: _Optional[int] = ..., trigger_character: _Optional[str] = ...) -> None: ...

class CompletionResponse(_message.Message):
    __slots__ = ("items",)
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_completion_pb2.CompletionItem]
    def __init__(self, items: _Optional[_Iterable[_Union[_completion_pb2.CompletionItem, _Mapping]]] = ...) -> None: ...



from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DriverListResponse(_message.Message):
    __slots__ = ("drivers",)
    DRIVERS_FIELD_NUMBER: _ClassVar[int]
    drivers: _containers.RepeatedCompositeFieldContainer[DriverInfo]
    def __init__(self, drivers: _Optional[_Iterable[_Union[DriverInfo, _Mapping]]] = ...) -> None: ...

class DriverInfo(_message.Message):
    __slots__ = ("driver_id", "display_name", "version", "file_size_bytes")
    DRIVER_ID_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    FILE_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    driver_id: str
    display_name: str
    version: str
    file_size_bytes: int
    def __init__(self, driver_id: _Optional[str] = ..., display_name: _Optional[str] = ..., version: _Optional[str] = ..., file_size_bytes: _Optional[int] = ...) -> None: ...

class DriverRequest(_message.Message):
    __slots__ = ("driver_id",)
    DRIVER_ID_FIELD_NUMBER: _ClassVar[int]
    driver_id: str
    def __init__(self, driver_id: _Optional[str] = ...) -> None: ...

class FileChunk(_message.Message):
    __slots__ = ("content", "offset")
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    content: bytes
    offset: int
    def __init__(self, content: _Optional[bytes] = ..., offset: _Optional[int] = ...) -> None: ...



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'connection/connection_state.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!connection/connection_state.proto\x12\x19ares.datamodel.connection"\x1a\n\x0cStateRequest\x12\n\n\x02id\x18\x01 \x01(\t"W\n\rStateResponse\x12/\n\x05state\x18\x01 \x01(\x0e2 .ares.datamodel.connection.State\x12\x15\n\rstate_message\x18\x02 \x01(\t*C\n\x05State\x12\x15\n\x11UNSPECIFIED_STATE\x10\x00\x12\n\n\x06ACTIVE\x10\x01\x12\x0c\n\x08INACTIVE\x10\x02\x12\t\n\x05ERROR\x10\x03b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'connection.connection_state_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_STATE']._serialized_start = 181
    _globals['_STATE']._serialized_end = 248
    _globals['_STATEREQUEST']._serialized_start = 64
    _globals['_STATEREQUEST']._serialized_end = 90
    _globals['_STATERESPONSE']._serialized_start = 92
    _globals['_STATERESPONSE']._serialized_end = 179


import ares_struct_pb2 as _ares_struct_pb2
import ares_data_type_pb2 as _ares_data_type_pb2
import ares_outcome_enum_pb2 as _ares_outcome_enum_pb2
import request_metadata_pb2 as _request_metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PlanningRequest(_message.Message):
    __slots__ = ("planning_parameters", "adapter_settings", "analysis_results", "metadata")
    PLANNING_PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    ADAPTER_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    ANALYSIS_RESULTS_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    planning_parameters: _containers.RepeatedCompositeFieldContainer[PlanningParameter]
    adapter_settings: _ares_struct_pb2.AresStruct
    analysis_results: _containers.RepeatedScalarFieldContainer[float]
    metadata: _request_metadata_pb2.RequestMetadata
    def __init__(self, planning_parameters: _Optional[_Iterable[_Union[PlanningParameter, _Mapping]]] = ..., adapter_settings: _Optional[_Union[_ares_struct_pb2.AresStruct, _Mapping]] = ..., analysis_results: _Optional[_Iterable[float]] = ..., metadata: _Optional[_Union[_request_metadata_pb2.RequestMetadata, _Mapping]] = ...) -> None: ...

class PlanningResponse(_message.Message):
    __slots__ = ("planned_parameters", "planning_outcome", "error_string")
    PLANNED_PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    PLANNING_OUTCOME_FIELD_NUMBER: _ClassVar[int]
    ERROR_STRING_FIELD_NUMBER: _ClassVar[int]
    planned_parameters: _containers.RepeatedCompositeFieldContainer[PlannedParameter]
    planning_outcome: _ares_outcome_enum_pb2.Outcome
    error_string: str
    def __init__(self, planned_parameters: _Optional[_Iterable[_Union[PlannedParameter, _Mapping]]] = ..., planning_outcome: _Optional[_Union[_ares_outcome_enum_pb2.Outcome, str]] = ..., error_string: _Optional[str] = ...) -> None: ...

class PlanningParameter(_message.Message):
    __slots__ = ("parameter_name", "minimum_value", "maximum_value", "minimum_precision", "parameter_history", "data_type", "metadata", "is_planned", "is_result", "planner_name", "initial_value")
    PARAMETER_NAME_FIELD_NUMBER: _ClassVar[int]
    MINIMUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    MAXIMUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    MINIMUM_PRECISION_FIELD_NUMBER: _ClassVar[int]
    PARAMETER_HISTORY_FIELD_NUMBER: _ClassVar[int]
    DATA_TYPE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    IS_PLANNED_FIELD_NUMBER: _ClassVar[int]
    IS_RESULT_FIELD_NUMBER: _ClassVar[int]
    PLANNER_NAME_FIELD_NUMBER: _ClassVar[int]
    INITIAL_VALUE_FIELD_NUMBER: _ClassVar[int]
    parameter_name: str
    minimum_value: float
    maximum_value: float
    minimum_precision: float
    parameter_history: _containers.RepeatedCompositeFieldContainer[ParameterHistoryInfo]
    data_type: _ares_data_type_pb2.AresDataType
    metadata: PlannerMetadata
    is_planned: bool
    is_result: bool
    planner_name: str
    initial_value: _ares_struct_pb2.AresValue
    def __init__(self, parameter_name: _Optional[str] = ..., minimum_value: _Optional[float] = ..., maximum_value: _Optional[float] = ..., minimum_precision: _Optional[float] = ..., parameter_history: _Optional[_Iterable[_Union[ParameterHistoryInfo, _Mapping]]] = ..., data_type: _Optional[_Union[_ares_data_type_pb2.AresDataType, str]] = ..., metadata: _Optional[_Union[PlannerMetadata, _Mapping]] = ..., is_planned: bool = ..., is_result: bool = ..., planner_name: _Optional[str] = ..., initial_value: _Optional[_Union[_ares_struct_pb2.AresValue, _Mapping]] = ...) -> None: ...

class ParameterHistoryInfo(_message.Message):
    __slots__ = ("planned_value", "achieved_value")
    PLANNED_VALUE_FIELD_NUMBER: _ClassVar[int]
    ACHIEVED_VALUE_FIELD_NUMBER: _ClassVar[int]
    planned_value: _ares_struct_pb2.AresValue
    achieved_value: _ares_struct_pb2.AresValue
    def __init__(self, planned_value: _Optional[_Union[_ares_struct_pb2.AresValue, _Mapping]] = ..., achieved_value: _Optional[_Union[_ares_struct_pb2.AresValue, _Mapping]] = ...) -> None: ...

class PlannedParameter(_message.Message):
    __slots__ = ("parameter_name", "parameter_value", "metadata")
    PARAMETER_NAME_FIELD_NUMBER: _ClassVar[int]
    PARAMETER_VALUE_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    parameter_name: str
    parameter_value: _ares_struct_pb2.AresValue
    metadata: PlannerMetadata
    def __init__(self, parameter_name: _Optional[str] = ..., parameter_value: _Optional[_Union[_ares_struct_pb2.AresValue, _Mapping]] = ..., metadata: _Optional[_Union[PlannerMetadata, _Mapping]] = ...) -> None: ...

class PlannerMetadata(_message.Message):
    __slots__ = ("metadata_name",)
    METADATA_NAME_FIELD_NUMBER: _ClassVar[int]
    metadata_name: str
    def __init__(self, metadata_name: _Optional[str] = ...) -> None: ...



import ares_data_schema_pb2 as _ares_data_schema_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SignatureParameter(_message.Message):
    __slots__ = ("name", "schema", "documentation")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    DOCUMENTATION_FIELD_NUMBER: _ClassVar[int]
    name: str
    schema: _ares_data_schema_pb2.AresValueSchema
    documentation: str
    def __init__(self, name: _Optional[str] = ..., schema: _Optional[_Union[_ares_data_schema_pb2.AresValueSchema, _Mapping]] = ..., documentation: _Optional[str] = ...) -> None: ...

class FunctionSignature(_message.Message):
    __slots__ = ("label", "documentation", "parameters", "return_schema")
    LABEL_FIELD_NUMBER: _ClassVar[int]
    DOCUMENTATION_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    RETURN_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    label: str
    documentation: str
    parameters: _containers.RepeatedCompositeFieldContainer[SignatureParameter]
    return_schema: _ares_data_schema_pb2.AresValueSchema
    def __init__(self, label: _Optional[str] = ..., documentation: _Optional[str] = ..., parameters: _Optional[_Iterable[_Union[SignatureParameter, _Mapping]]] = ..., return_schema: _Optional[_Union[_ares_data_schema_pb2.AresValueSchema, _Mapping]] = ...) -> None: ...

class SignatureHelp(_message.Message):
    __slots__ = ("signatures", "active_signature", "active_parameter")
    SIGNATURES_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    signatures: _containers.RepeatedCompositeFieldContainer[FunctionSignature]
    active_signature: int
    active_parameter: int
    def __init__(self, signatures: _Optional[_Iterable[_Union[FunctionSignature, _Mapping]]] = ..., active_signature: _Optional[int] = ..., active_parameter: _Optional[int] = ...) -> None: ...



from scripting import function_invocation_kind_pb2 as _function_invocation_kind_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ScriptFunctionInvocation(_message.Message):
    __slots__ = ("order", "function_id", "function_name", "expression", "line", "column", "kind")
    ORDER_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_ID_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_NAME_FIELD_NUMBER: _ClassVar[int]
    EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    LINE_FIELD_NUMBER: _ClassVar[int]
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    KIND_FIELD_NUMBER: _ClassVar[int]
    order: int
    function_id: str
    function_name: str
    expression: str
    line: int
    column: int
    kind: _function_invocation_kind_pb2.FunctionInvocationKind
    def __init__(self, order: _Optional[int] = ..., function_id: _Optional[str] = ..., function_name: _Optional[str] = ..., expression: _Optional[str] = ..., line: _Optional[int] = ..., column: _Optional[int] = ..., kind: _Optional[_Union[_function_invocation_kind_pb2.FunctionInvocationKind, str]] = ...) -> None: ...



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/script_symbol_metadata.proto')
_sym_db = _symbol_database.Default()
from .. import ares_data_schema_pb2 as ares__data__schema__pb2
from .. import ares_struct_pb2 as ares__struct__pb2
from ..scripting import symbol_kind_pb2 as scripting_dot_symbol__kind__pb2
from ..scripting import symbol_tag_pb2 as scripting_dot_symbol__tag__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&scripting/script_symbol_metadata.proto\x12\x18ares.datamodel.scripting\x1a\x16ares_data_schema.proto\x1a\x11ares_struct.proto\x1a\x1bscripting/symbol_kind.proto\x1a\x1ascripting/symbol_tag.proto"\xb2\x05\n\x14ScriptSymbolMetadata\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x1e\n\x11parent_identifier\x18\x02 \x01(\tH\x01\x88\x01\x01\x122\n\x04kind\x18\x03 \x01(\x0e2$.ares.datamodel.scripting.SymbolKind\x121\n\x04tags\x18\x08 \x03(\x0e2#.ares.datamodel.scripting.SymbolTag\x12\x13\n\x06detail\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x1a\n\rdocumentation\x18\x05 \x01(\tH\x03\x88\x01\x01\x12P\n\x0bvalue_shape\x18\x06 \x01(\x0b29.ares.datamodel.scripting.ScriptSymbolMetadata.ValueShapeH\x00\x12V\n\x0efunction_shape\x18\x07 \x01(\x0b2<.ares.datamodel.scripting.ScriptSymbolMetadata.FunctionShapeH\x00\x1ag\n\nValueShape\x12/\n\x06schema\x18\x01 \x01(\x0b2\x1f.ares.datamodel.AresValueSchema\x12(\n\x05value\x18\x02 \x01(\x0b2\x19.ares.datamodel.AresValue\x1a\x7f\n\rFunctionShape\x126\n\x0cinput_schema\x18\x01 \x01(\x0b2 .ares.datamodel.AresStructSchema\x126\n\routput_schema\x18\x02 \x01(\x0b2\x1f.ares.datamodel.AresValueSchemaB\x07\n\x05shapeB\x14\n\x12_parent_identifierB\t\n\x07_detailB\x10\n\x0e_documentationb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.script_symbol_metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_SCRIPTSYMBOLMETADATA']._serialized_start = 169
    _globals['_SCRIPTSYMBOLMETADATA']._serialized_end = 859
    _globals['_SCRIPTSYMBOLMETADATA_VALUESHAPE']._serialized_start = 567
    _globals['_SCRIPTSYMBOLMETADATA_VALUESHAPE']._serialized_end = 670
    _globals['_SCRIPTSYMBOLMETADATA_FUNCTIONSHAPE']._serialized_start = 672
    _globals['_SCRIPTSYMBOLMETADATA_FUNCTIONSHAPE']._serialized_end = 799


from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class QuantityType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    QUANTITY_TYPE_UNSPECIFIED: _ClassVar[QuantityType]
    LENGTH: _ClassVar[QuantityType]
    AREA: _ClassVar[QuantityType]
    VOLUME: _ClassVar[QuantityType]
    DURATION: _ClassVar[QuantityType]
    SPEED: _ClassVar[QuantityType]
    ACCELERATION: _ClassVar[QuantityType]
    FREQUENCY: _ClassVar[QuantityType]
    MASS: _ClassVar[QuantityType]
    AMOUNT_OF_SUBSTANCE: _ClassVar[QuantityType]
    MOLAR_MASS: _ClassVar[QuantityType]
    TEMPERATURE: _ClassVar[QuantityType]
    TEMPERATURE_DELTA: _ClassVar[QuantityType]
    PRESSURE: _ClassVar[QuantityType]
    MOLARITY: _ClassVar[QuantityType]
    MASS_CONCENTRATION: _ClassVar[QuantityType]
    VOLUME_CONCENTRATION: _ClassVar[QuantityType]
    MASS_FRACTION: _ClassVar[QuantityType]
    DENSITY: _ClassVar[QuantityType]
    DYNAMIC_VISCOSITY: _ClassVar[QuantityType]
    KINEMATIC_VISCOSITY: _ClassVar[QuantityType]
    VOLUME_FLOW: _ClassVar[QuantityType]
    MASS_FLOW: _ClassVar[QuantityType]
    MASS_FLUX: _ClassVar[QuantityType]
    ROTATIONAL_ACCELERATION: _ClassVar[QuantityType]
    ROTATIONAL_SPEED: _ClassVar[QuantityType]
    ENERGY: _ClassVar[QuantityType]
    POWER: _ClassVar[QuantityType]
    HEAT_FLUX: _ClassVar[QuantityType]
    SPECIFIC_ENERGY: _ClassVar[QuantityType]
    THERMAL_CONDUCTIVITY: _ClassVar[QuantityType]
    FORCE: _ClassVar[QuantityType]
    FORCE_PER_LENGTH: _ClassVar[QuantityType]
    RATIO: _ClassVar[QuantityType]
    IRRADIANCE: _ClassVar[QuantityType]
    ILLUMINANCE: _ClassVar[QuantityType]
    TEMPERATURE_GRADIENT: _ClassVar[QuantityType]
    ELECTRIC_POTENTIAL: _ClassVar[QuantityType]
    ELECTRIC_CURRENT: _ClassVar[QuantityType]
    ELECTRIC_RESISTANCE: _ClassVar[QuantityType]
    ELECTRIC_CONDUCTIVITY: _ClassVar[QuantityType]
    ELECTRIC_CONDUCTANCE: _ClassVar[QuantityType]
    RATIO_CHANGE_RATE: _ClassVar[QuantityType]
    RELATIVE_HUMIDITY: _ClassVar[QuantityType]
QUANTITY_TYPE_UNSPECIFIED: QuantityType
LENGTH: QuantityType
AREA: QuantityType
VOLUME: QuantityType
DURATION: QuantityType
SPEED: QuantityType
ACCELERATION: QuantityType
FREQUENCY: QuantityType
MASS: QuantityType
AMOUNT_OF_SUBSTANCE: QuantityType
MOLAR_MASS: QuantityType
TEMPERATURE: QuantityType
TEMPERATURE_DELTA: QuantityType
PRESSURE: QuantityType
MOLARITY: QuantityType
MASS_CONCENTRATION: QuantityType
VOLUME_CONCENTRATION: QuantityType
MASS_FRACTION: QuantityType
DENSITY: QuantityType
DYNAMIC_VISCOSITY: QuantityType
KINEMATIC_VISCOSITY: QuantityType
VOLUME_FLOW: QuantityType
MASS_FLOW: QuantityType
MASS_FLUX: QuantityType
ROTATIONAL_ACCELERATION: QuantityType
ROTATIONAL_SPEED: QuantityType
ENERGY: QuantityType
POWER: QuantityType
HEAT_FLUX: QuantityType
SPECIFIC_ENERGY: QuantityType
THERMAL_CONDUCTIVITY: QuantityType
FORCE: QuantityType
FORCE_PER_LENGTH: QuantityType
RATIO: QuantityType
IRRADIANCE: QuantityType
ILLUMINANCE: QuantityType
TEMPERATURE_GRADIENT: QuantityType
ELECTRIC_POTENTIAL: QuantityType
ELECTRIC_CURRENT: QuantityType
ELECTRIC_RESISTANCE: QuantityType
ELECTRIC_CONDUCTIVITY: QuantityType
ELECTRIC_CONDUCTANCE: QuantityType
RATIO_CHANGE_RATE: QuantityType
RELATIVE_HUMIDITY: QuantityType



from __future__ import annotations

import ast
import collections
import collections.abc
import typing
from ast import AST
from typing import TYPE_CHECKING, Any, Callable, NamedTuple, Sequence, Union

try:
    from ast import unparse as astunparse
except ImportError:
    from astunparse import unparse as astunparse  # type: ignore[no-redef]

if TYPE_CHECKING:
    from collections.abc import MutableSet

    Node = Union[AST, Sequence[AST]]


class Replacement(NamedTuple):
    """A mapping from an old import to a new import."""

    old: str
    new: str


def _is_iterable(value: Any) -> bool:  # type: ignore[misc]
    """Return whether `value` is a non-string iterable.

    Examples
    --------
    >>> _is_iterable(1)
    False
    >>> _is_iterable([1])
    True
    >>> _is_iterable("a")
    False
    >>> _is_iterable(["a"])
    True
    """
    return not isinstance(value, str) and isinstance(value, collections.abc.Iterable)


def matches(value: Node, pattern: Node) -> bool:
    """Check whether `value` matches `pattern`.

    Parameters
    ----------
    value
        Instance to match
    pattern
        Pattern to match against

    Returns
    -------
    bool
        Whether `value` matches `pattern`

    Examples
    --------
    >>> import ast
    >>> node = ast.parse("x = 1")
    >>> pattern = node
    >>> matches(node, pattern)
    True
    >>> node = ast.parse("x = 2")
    >>> matches(node, pattern)
    False
    """
    # types must match exactly
    if type(value) is not type(pattern):
        return False

    # recur into non-str sequences
    if _is_iterable(value) and _is_iterable(pattern):
        assert isinstance(
            value, collections.abc.Iterable
        ), f"value is not a non-str iterable {type(value).__name__}"
        assert isinstance(
            pattern, collections.abc.Iterable
        ), f"pattern is not a non-str iterable {type(pattern).__name__}"
        return all(map(matches, value, pattern))

    # primitive value, such as None, True, False, etc.
    if not isinstance(value, AST) and not isinstance(pattern, AST):
        return value == pattern

    assert isinstance(value, AST), f"value is not an AST node: {type(value).__name__}"
    assert isinstance(
        pattern, AST
    ), f"pattern is not an AST node: {type(pattern).__name__}"

    return all(
        matches(getattr(value, field), getattr(pattern, field))
        for field in pattern._fields
        if hasattr(pattern, field)
    )


ASTTransform = Callable[[AST], AST]


class ASTRewriter:
    """AST pattern matching to enable rewrite rules."""

    def __init__(self) -> None:
        self.funcs: list[tuple[AST, ASTTransform]] = []

    def register(self, pattern: AST) -> Callable[[ASTTransform], ASTTransform]:
        """Register a callable to rewrite `pattern`."""

        def wrapper(f: ASTTransform) -> ASTTransform:
            self.funcs.append((pattern, f))
            return f

        return wrapper

    def rewrite(self, node: AST) -> AST:
        """Rewrite `node` if it matches a registered pattern.

        Examples
        --------
        >>> rewriter = ASTRewriter()
        >>> pattern = ast.parse("x = 1")
        >>> @rewriter.register(pattern)
        ... def make_it_two(node):
        ...     return ast.parse("x = 2")
        ...
        >>> node = ast.parse("x = 1")
        >>> print(astunparse(node))  # doctest: +NORMALIZE_WHITESPACE
        x = 1
        >>> rewritten = rewriter.rewrite(node)
        >>> print(astunparse(rewritten))  # doctest: +NORMALIZE_WHITESPACE
        x = 2
        """
        try:
            return next(
                func(node) for pattern, func in self.funcs if matches(node, pattern)
            )
        except StopIteration:
            return node


def build_rewrites(
    proto: str, dep: str, *, is_public: bool = False
) -> list[Replacement]:
    """Construct a replacement import for `dep`.

    Parameters
    ----------
    proto
        The dependent protobuf path.
    dep
        The `name` field of a `FileDescriptorProto` message, stripped
        of its ``.proto`` suffix.
    is_public
        Whether the dependency is public. We generate star imports like protoc
        if this is `True`.

    Returns
    -------
    list[Replacement]
        A list of mapping from old code to new code

    Examples
    --------
    >>> from pprint import pprint
    >>> proto = "a/b/c"
    >>> dep = "foo/bar"
    >>> pprint(build_rewrites(proto, dep))
    [Replacement(old='from foo import bar_pb2 as foo_dot_bar__pb2', new='from ...foo import bar_pb2 as foo_dot_bar__pb2'),
     Replacement(old='import foo.bar_pb2', new='from ... import foo')]
    """
    parts = dep.split("/")

    *import_parts, part = parts

    # compute the number of dots needed to get to the package root relative to
    # the dependent proto file
    num_leading_dots = proto.count("/") + 1
    leading_dots = "." * num_leading_dots
    last_part = "__".join(f"{part}_pb2".split("_"))

    from_ = ".".join(import_parts)

    if not import_parts:
        # underscores are doubled by codegen
        old = f"import {part}_pb2 as {last_part}"
        new = f"from {leading_dots} import {part}_pb2 as {last_part}"
    else:
        fake_dotted_path = "_dot_".join(
            part.replace("_", "__") for part in import_parts
        )
        as_ = f"{fake_dotted_path}_dot_{last_part}"

        old = f"from {from_} import {part}_pb2 as {as_}"
        new = f"from {leading_dots}{from_} import {part}_pb2 as {as_}"

    replacements = [
        Replacement(old=old, new=new),
        Replacement(
            old=f"import {'.'.join(parts)}_pb2",
            new=(
                f"from {leading_dots or '.'} import {parts[0]}"
                + "_pb2" * (not import_parts)
            ),
        ),
    ]
    if is_public:
        replacements.append(
            Replacement(
                old=f"from {'.'.join(parts)}_pb2 import *",
                new=f"from {'.' * (num_leading_dots - 1)}{from_}.{part}_pb2 import *",
            ),
        )
    return replacements


class ImportNodeTransformer(ast.NodeTransformer):
    """A NodeTransformer to apply rewrite rules."""

    def __init__(self, ast_rewriter: ASTRewriter) -> None:
        self.ast_rewriter = ast_rewriter
        # track the results we've produced to avoid duplication of imports
        self.seen: MutableSet[str] = set()

    def visit_Import(self, node: ast.AST) -> AST | None:
        result = self.ast_rewriter.rewrite(node)
        code = astunparse(result)
        if code not in self.seen:
            self.seen.add(code)
            return result
        return None

    visit_ImportFrom = visit_Import


class ASTImportRewriter:
    def __init__(self) -> None:
        self.node_transformer = ImportNodeTransformer(ASTRewriter())

    def register_rewrite(self, replacement: Replacement) -> None:
        """Register a rewrite rule for turning `old` into `new`."""
        (old_node,) = typing.cast(ast.Module, ast.parse(replacement.old)).body
        (new_node,) = typing.cast(ast.Module, ast.parse(replacement.new)).body

        def _rewrite(_: AST, repl: AST = new_node) -> AST:
            return repl

        funcs = self.node_transformer.ast_rewriter.funcs
        if all(not matches(old_node, pat) for pat, _ in funcs):
            self.node_transformer.ast_rewriter.register(old_node)(_rewrite)
        assert (
            sum(matches(old_node, pat) for pat, _ in funcs) == 1
        ), f"more than one rewrite rule found for pattern `{replacement.old}`"

    def rewrite(self, src: str) -> str:
        self.node_transformer.seen.clear()
        return astunparse(self.node_transformer.visit(ast.parse(src)))



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'planning/remote/ares_remote_planner_service.proto')
_sym_db = _symbol_database.Default()
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from ...planning import plan_pb2 as planning_dot_plan__pb2
from ...planning import planner_service_capabilities_pb2 as planning_dot_planner__service__capabilities__pb2
from ...connection import connection_state_pb2 as connection_dot_connection__state__pb2
from ...connection import connection_status_pb2 as connection_dot_connection__status__pb2
from ...connection import connection_info_pb2 as connection_dot_connection__info__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1planning/remote/ares_remote_planner_service.proto\x12\x1eares.datamodel.planning.remote\x1a\x1bgoogle/protobuf/empty.proto\x1a\x13planning/plan.proto\x1a+planning/planner_service_capabilities.proto\x1a!connection/connection_state.proto\x1a"connection/connection_status.proto\x1a connection/connection_info.proto2\xdb\x03\n\x18AresRemotePlannerService\x12[\n\x04Plan\x12(.ares.datamodel.planning.PlanningRequest\x1a).ares.datamodel.planning.PlanningResponse\x12l\n\x1dGetPlannerServiceCapabilities\x12\x16.google.protobuf.Empty\x1a3.ares.datamodel.planning.PlannerServiceCapabilities\x12L\n\x08GetState\x12\x16.google.protobuf.Empty\x1a(.ares.datamodel.connection.StateResponse\x12Z\n\x13GetConnectionStatus\x12\x16.google.protobuf.Empty\x1a+.ares.datamodel.connection.ConnectionStatus\x12J\n\x07GetInfo\x12\x16.google.protobuf.Empty\x1a\'.ares.datamodel.connection.InfoResponseb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'planning.remote.ares_remote_planner_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_ARESREMOTEPLANNERSERVICE']._serialized_start = 286
    _globals['_ARESREMOTEPLANNERSERVICE']._serialized_end = 761


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/signature_help.proto')
_sym_db = _symbol_database.Default()
from .. import ares_data_schema_pb2 as ares__data__schema__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1escripting/signature_help.proto\x12\x18ares.datamodel.scripting\x1a\x16ares_data_schema.proto"\x81\x01\n\x12SignatureParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\x06schema\x18\x02 \x01(\x0b2\x1f.ares.datamodel.AresValueSchema\x12\x1a\n\rdocumentation\x18\x03 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_documentation"\xca\x01\n\x11FunctionSignature\x12\r\n\x05label\x18\x01 \x01(\t\x12\x1a\n\rdocumentation\x18\x02 \x01(\tH\x00\x88\x01\x01\x12@\n\nparameters\x18\x03 \x03(\x0b2,.ares.datamodel.scripting.SignatureParameter\x126\n\rreturn_schema\x18\x04 \x01(\x0b2\x1f.ares.datamodel.AresValueSchemaB\x10\n\x0e_documentation"\xb8\x01\n\rSignatureHelp\x12?\n\nsignatures\x18\x01 \x03(\x0b2+.ares.datamodel.scripting.FunctionSignature\x12\x1d\n\x10active_signature\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x1d\n\x10active_parameter\x18\x03 \x01(\x05H\x01\x88\x01\x01B\x13\n\x11_active_signatureB\x13\n\x11_active_parameterb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.signature_help_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_SIGNATUREPARAMETER']._serialized_start = 85
    _globals['_SIGNATUREPARAMETER']._serialized_end = 214
    _globals['_FUNCTIONSIGNATURE']._serialized_start = 217
    _globals['_FUNCTIONSIGNATURE']._serialized_end = 419
    _globals['_SIGNATUREHELP']._serialized_start = 422
    _globals['_SIGNATUREHELP']._serialized_end = 606


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'visualizing/visualizer_settings.proto')
_sym_db = _symbol_database.Default()
from .. import ares_struct_pb2 as ares__struct__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%visualizing/visualizer_settings.proto\x12\x18ares.datamodel.analyzing\x1a\x11ares_struct.proto"Y\n\x12VisualizerSettings\x12\x15\n\rvisualizer_id\x18\x01 \x01(\t\x12,\n\x08settings\x18\x02 \x01(\x0b2\x1a.ares.datamodel.AresStructb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'visualizing.visualizer_settings_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_VISUALIZERSETTINGS']._serialized_start = 86
    _globals['_VISUALIZERSETTINGS']._serialized_end = 175


from __future__ import annotations

import abc
import fnmatch
import itertools
import re
from pathlib import Path
from typing import TYPE_CHECKING, Callable

from google.protobuf.descriptor_pb2 import FileDescriptorSet

from .rewrite import ASTImportRewriter, build_rewrites

if TYPE_CHECKING:
    from collections.abc import Iterable, Sequence

_PROTO_SUFFIX_PATTERN = re.compile(r"^(.+)\.proto$")


def _clean_proto_filename(name: str) -> str:
    """Remove the `.proto` suffix from `name`.

    Examples
    --------
    >>> _clean_proto_filename("a/b.proto")
    'a/b'
    >>> _clean_proto_filename("a/b-c.proto")
    'a/b_c'
    >>> _clean_proto_filename("a/b_c.proto")
    'a/b_c'
    """
    return _PROTO_SUFFIX_PATTERN.sub(r"\1", name).replace("-", "_")


def _should_ignore(fd_name: str, patterns: Sequence[str]) -> bool:
    """Return whether `fd_name` should be ignored according to `patterns`.

    Examples
    --------
    >>> fd_name = "google/protobuf/empty.proto"
    >>> pattern = "google/protobuf/*"
    >>> _should_ignore(fd_name, [pattern])
    True
    >>> fd_name = "foo/bar"
    >>> _should_ignore(fd_name, [pattern])
    False
    """
    return any(fnmatch.fnmatchcase(fd_name, pattern) for pattern in patterns)


class FileDescriptorSetGenerator(abc.ABC):
    """Base class that implements fixing imports."""

    @abc.abstractmethod
    def generate_file_descriptor_set_bytes(self) -> bytes:
        """Generate the bytes of a `FileDescriptorSet`."""

    def fix_imports(
        self,
        *,
        python_out: Path,
        create_package: bool,
        overwrite_callback: Callable[[Path, str], None],
        module_suffixes: Sequence[str],
        exclude_imports_glob: Sequence[str],
    ) -> None:
        """Fix imports from protoc/buf generated code."""
        fdset = FileDescriptorSet.FromString(self.generate_file_descriptor_set_bytes())

        has_pyi = any(suffix.endswith(".pyi") for suffix in module_suffixes)
        for fd in fdset.file:
            if _should_ignore(fd.name, exclude_imports_glob):
                continue

            fd_name = _clean_proto_filename(fd.name)
            rewriter = ASTImportRewriter()
            # services live outside of the corresponding generated Python
            # module, but they import it so we register a rewrite for the
            # current proto as a dependency of itself to handle the case
            # of services
            for repl in build_rewrites(fd_name, fd_name, is_public=False):
                rewriter.register_rewrite(repl)

            # register proto import rewrites

            # construct a frozenset for dependencies to check whether need to
            # rewrite using star imports
            public_deps = frozenset(fd.public_dependency)

            for i, dep in enumerate(map(_clean_proto_filename, fd.dependency)):
                if _should_ignore(dep, exclude_imports_glob):
                    continue

                dep_name = _clean_proto_filename(dep)
                for repl in build_rewrites(
                    fd_name, dep_name, is_public=i in public_deps
                ):
                    rewriter.register_rewrite(repl)

            for suffix in module_suffixes:
                python_file = python_out.joinpath(f"{fd_name}{suffix}")
                try:
                    raw_code = python_file.read_text()
                except FileNotFoundError:
                    pass
                else:
                    new_code = rewriter.rewrite(raw_code)
                    overwrite_callback(python_file, new_code)

        if create_package:
            # recursively create packages
            for dir_entry in itertools.chain([python_out], python_out.rglob("*")):
                if dir_entry.is_dir() and "__pycache__" not in dir_entry.parts:
                    dir_entry.joinpath("__init__.py").touch(exist_ok=True)
                    if has_pyi:
                        _create_pyi_init(dir_entry)


def _create_pyi_init(root: Path) -> None:
    # use a dictionary to preserve order while deduplicating
    lines_to_write = {
        f"from . import {path.stem}\n": None
        for path in sorted(root.glob("*"))
        if path.stem not in ("__init__", "__pycache__")
        if path.suffix == ".pyi" or path.is_dir()
    }
    path = root.joinpath("__init__.pyi")

    if not path.exists():
        path.write_text("".join(lines_to_write))
    else:
        with path.open(mode="r") as f:
            for line in f:
                lines_to_write.pop(line, None)
        with path.open(mode="a") as f:
            f.writelines(lines_to_write)


# Custom generator that works with protoletariat
class CustomProtoc(FileDescriptorSetGenerator):
    def __init__(
        self,
        descriptor_path: Path
    ) -> None:
        self.descriptor_path = descriptor_path

    def generate_file_descriptor_set_bytes(self) -> bytes:
        return self.descriptor_path.read_bytes()



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/navigation.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ascripting/navigation.proto\x12\x18ares.datamodel.scripting"]\n\x0bScriptRange\x12\x12\n\nstart_line\x18\x01 \x01(\x05\x12\x14\n\x0cstart_column\x18\x02 \x01(\x05\x12\x10\n\x08end_line\x18\x03 \x01(\x05\x12\x12\n\nend_column\x18\x04 \x01(\x05"\x90\x01\n\x0eSymbolLocation\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x1e\n\x11parent_identifier\x18\x02 \x01(\tH\x00\x88\x01\x01\x124\n\x05range\x18\x03 \x01(\x0b2%.ares.datamodel.scripting.ScriptRangeB\x14\n\x12_parent_identifierb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.navigation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_SCRIPTRANGE']._serialized_start = 56
    _globals['_SCRIPTRANGE']._serialized_end = 149
    _globals['_SYMBOLLOCATION']._serialized_start = 152
    _globals['_SYMBOLLOCATION']._serialized_end = 296


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'planning/planner_config.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dplanning/planner_config.proto\x12\x17ares.datamodel.planning"P\n\rPlannerConfig\x12\x16\n\tunique_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\tB\x0c\n\n_unique_idb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'planning.planner_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_PLANNERCONFIG']._serialized_start = 58
    _globals['_PLANNERCONFIG']._serialized_end = 138


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'templates/output_metadata.proto')
_sym_db = _symbol_database.Default()
from .. import ares_data_schema_pb2 as ares__data__schema__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ftemplates/output_metadata.proto\x12\x18ares.datamodel.templates\x1a\x16ares_data_schema.proto"\x90\x01\n\x0eOutputMetadata\x12\x16\n\tunique_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x124\n\x0bdata_schema\x18\x02 \x01(\x0b2\x1f.ares.datamodel.AresValueSchema\x12\x13\n\x0bdescription\x18\x03 \x01(\t\x12\r\n\x05index\x18\x04 \x01(\x03B\x0c\n\n_unique_idb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'templates.output_metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_OUTPUTMETADATA']._serialized_start = 86
    _globals['_OUTPUTMETADATA']._serialized_end = 230


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'ares_devices.proto')
_sym_db = _symbol_database.Default()
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from .templates import command_metadata_pb2 as templates_dot_command__metadata__pb2
from .templates import command_template_pb2 as templates_dot_command__template__pb2
from .device import device_execution_result_pb2 as device_dot_device__execution__result__pb2
from .device import remote_device_config_pb2 as device_dot_remote__device__config__pb2
from .device import device_status_pb2 as device_dot_device__status__pb2
from .device import device_info_pb2 as device_dot_device__info__pb2
from .device import device_config_pb2 as device_dot_device__config__pb2
from .device import device_settings_pb2 as device_dot_device__settings__pb2
from .device import device_polling_settings_pb2 as device_dot_device__polling__settings__pb2
from .device import device_description_pb2 as device_dot_device__description__pb2
from . import ares_struct_pb2 as ares__struct__pb2
from . import ares_data_schema_pb2 as ares__data__schema__pb2
from .device import device_logging_settings_pb2 as device_dot_device__logging__settings__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12ares_devices.proto\x12\x14ares.services.device\x1a\x1bgoogle/protobuf/empty.proto\x1a templates/command_metadata.proto\x1a templates/command_template.proto\x1a$device/device_execution_result.proto\x1a!device/remote_device_config.proto\x1a\x1adevice/device_status.proto\x1a\x18device/device_info.proto\x1a\x1adevice/device_config.proto\x1a\x1cdevice/device_settings.proto\x1a$device/device_polling_settings.proto\x1a\x1fdevice/device_description.proto\x1a\x11ares_struct.proto\x1a\x16ares_data_schema.proto\x1a$device/device_logging_settings.proto"R\n\x17ListAresDevicesResponse\x127\n\x0cares_devices\x18\x01 \x03(\x0b2!.ares.datamodel.device.DeviceInfo"S\n\x1dListAresRemoteDevicesResponse\x122\n\x07devices\x18\x01 \x03(\x0b2!.ares.datamodel.device.DeviceInfo"5\n\x1dListServerSerialPortsResponse\x12\x14\n\x0cserial_ports\x18\x01 \x03(\t",\n\x17CommandMetadatasRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"X\n\x18CommandMetadatasResponse\x12<\n\tmetadatas\x18\x01 \x03(\x0b2).ares.datamodel.templates.CommandMetadata"(\n\x13DeviceStatusRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"u\n\x18DeviceStateStreamRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12F\n\x10polling_settings\x18\x02 \x01(\x0b2,.ares.datamodel.device.DevicePollingSettings"\'\n\x12DeviceStateRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"@\n\x13DeviceStateResponse\x12)\n\x05state\x18\x01 \x01(\x0b2\x1a.ares.datamodel.AresStruct"-\n\x18DeviceStateSchemaRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"M\n\x19DeviceStateSchemaResponse\x120\n\x06schema\x18\x01 \x01(\x0b2 .ares.datamodel.AresStructSchema"0\n\x1bDeviceLoggerSettingsRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"V\n\x15DeviceLoggersResponse\x12=\n\x07loggers\x18\x01 \x03(\x0b2,.ares.datamodel.device.DeviceLoggingSettings"&\n\x11DeviceInfoRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"*\n\x15DeviceSettingsRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"*\n\x15DeviceActivateRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"*\n\x13DeviceConfigRequest\x12\x13\n\x0bdevice_type\x18\x01 \x01(\t"L\n\x14DeviceConfigResponse\x124\n\x07configs\x18\x01 \x03(\x0b2#.ares.datamodel.device.DeviceConfig"X\n\x1aRemoteDeviceConfigResponse\x12:\n\x07configs\x18\x01 \x03(\x0b2).ares.datamodel.device.RemoteDeviceConfig"3\n\x16AddRemoteDeviceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t"k\n\x17AddRemoteDeviceResponse\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x1a\n\rerror_message\x18\x03 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_error_message"d\n\x19UpdateRemoteDeviceRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x11\n\x04name\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03url\x18\x03 \x01(\tH\x01\x88\x01\x01B\x07\n\x05_nameB\x06\n\x04_url"[\n\x1aUpdateRemoteDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rerror_message\x18\x02 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_error_message".\n\x19RemoveRemoteDeviceRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"[\n\x1aRemoveRemoteDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rerror_message\x18\x02 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_error_message"x\n\x10AddDeviceRequest\x12\x13\n\x0bdevice_name\x18\x01 \x01(\t\x12\x13\n\x0bdriver_name\x18\x02 \x01(\t\x12:\n\rdevice_config\x18\x03 \x01(\x0b2#.ares.datamodel.device.DeviceConfig";\n\x11AddDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t"8\n\x1eAvailableDeviceDriversResponse\x12\x16\n\x0edevice_drivers\x18\x01 \x03(\t"@\n\x16GetDeviceConfigRequest\x12\x13\n\x0bdriver_name\x18\x01 \x01(\t\x12\x11\n\tdriver_id\x18\x02 \x01(\t"Y\n\x18AvailableDevicesResponse\x12=\n\x07devices\x18\x01 \x03(\x0b2,.ares.datamodel.device.AresDeviceDescription"e\n\x13UpdateDeviceRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12;\n\x0eupdated_config\x18\x02 \x01(\x0b2#.ares.datamodel.device.DeviceConfig"U\n\x14UpdateDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rerror_message\x18\x02 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_error_message"(\n\x13RemoveDeviceRequest\x12\x11\n\tdevice_id\x18\x01 \x01(\t"U\n\x14RemoveDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rerror_message\x18\x02 \x01(\tH\x00\x88\x01\x01B\x10\n\x0e_error_message2\x96\x16\n\x0bAresDevices\x12X\n\x0fListAresDevices\x12\x16.google.protobuf.Empty\x1a-.ares.services.device.ListAresDevicesResponse\x12[\n\rGetDeviceInfo\x12\'.ares.services.device.DeviceInfoRequest\x1a!.ares.datamodel.device.DeviceInfo\x12c\n\x14GetServerSerialPorts\x12\x16.google.protobuf.Empty\x1a3.ares.services.device.ListServerSerialPortsResponse\x12l\n\x0fGetDeviceStatus\x12).ares.services.device.DeviceStatusRequest\x1a..ares.datamodel.device.DeviceOperationalStatus\x12t\n\x13GetCommandMetadatas\x12-.ares.services.device.CommandMetadatasRequest\x1a..ares.services.device.CommandMetadatasResponse\x12i\n\x0eExecuteCommand\x12).ares.datamodel.templates.CommandTemplate\x1a,.ares.datamodel.device.DeviceExecutionResult\x12l\n\x13GetAllDeviceConfigs\x12).ares.services.device.DeviceConfigRequest\x1a*.ares.services.device.DeviceConfigResponse\x12O\n\x08Activate\x12+.ares.services.device.DeviceActivateRequest\x1a\x16.google.protobuf.Empty\x12R\n\x11SetDeviceSettings\x12%.ares.datamodel.device.DeviceSettings\x1a\x16.google.protobuf.Empty\x12\\\n\x11GetDeviceSettings\x12+.ares.services.device.DeviceSettingsRequest\x1a\x1a.ares.datamodel.AresStruct\x12e\n\x0eGetDeviceState\x12(.ares.services.device.DeviceStateRequest\x1a).ares.services.device.DeviceStateResponse\x12s\n\x14GetDeviceStateStream\x12..ares.services.device.DeviceStateStreamRequest\x1a).ares.services.device.DeviceStateResponse0\x01\x12w\n\x14GetDeviceStateSchema\x12..ares.services.device.DeviceStateSchemaRequest\x1a/.ares.services.device.DeviceStateSchemaResponse\x12_\n\x17SetDeviceLoggerSettings\x12,.ares.datamodel.device.DeviceLoggingSettings\x1a\x16.google.protobuf.Empty\x12z\n\x17GetDeviceLoggerSettings\x121.ares.services.device.DeviceLoggerSettingsRequest\x1a,.ares.datamodel.device.DeviceLoggingSettings\x12W\n\x10GetDeviceLoggers\x12\x16.google.protobuf.Empty\x1a+.ares.services.device.DeviceLoggersResponse\x12d\n\x15ListRemoteAresDevices\x12\x16.google.protobuf.Empty\x1a3.ares.services.device.ListAresRemoteDevicesResponse\x12n\n\x0fAddRemoteDevice\x12,.ares.services.device.AddRemoteDeviceRequest\x1a-.ares.services.device.AddRemoteDeviceResponse\x12w\n\x12RemoveRemoteDevice\x12/.ares.services.device.RemoveRemoteDeviceRequest\x1a0.ares.services.device.RemoveRemoteDeviceResponse\x12w\n\x12UpdateRemoteDevice\x12/.ares.services.device.UpdateRemoteDeviceRequest\x1a0.ares.services.device.UpdateRemoteDeviceResponse\x12f\n\x1aGetAllRemoteDevicesConfigs\x12\x16.google.protobuf.Empty\x1a0.ares.services.device.RemoteDeviceConfigResponse\x12`\n\rAddAresDevice\x12&.ares.services.device.AddDeviceRequest\x1a\'.ares.services.device.AddDeviceResponse\x12i\n\x10UpdateAresDevice\x12).ares.services.device.UpdateDeviceRequest\x1a*.ares.services.device.UpdateDeviceResponse\x12i\n\x10RemoveAresDevice\x12).ares.services.device.RemoveDeviceRequest\x1a*.ares.services.device.RemoveDeviceResponse\x12i\n\x19GetAvailableDeviceDrivers\x12\x16.google.protobuf.Empty\x1a4.ares.services.device.AvailableDeviceDriversResponse\x12k\n\x0fGetDeviceConfig\x12,.ares.services.device.GetDeviceConfigRequest\x1a*.ares.services.device.DeviceConfigResponse\x12`\n\x16GetAllAvailableDevices\x12\x16.google.protobuf.Empty\x1a..ares.services.device.AvailableDevicesResponseb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ares_devices_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_LISTARESDEVICESRESPONSE']._serialized_start = 478
    _globals['_LISTARESDEVICESRESPONSE']._serialized_end = 560
    _globals['_LISTARESREMOTEDEVICESRESPONSE']._serialized_start = 562
    _globals['_LISTARESREMOTEDEVICESRESPONSE']._serialized_end = 645
    _globals['_LISTSERVERSERIALPORTSRESPONSE']._serialized_start = 647
    _globals['_LISTSERVERSERIALPORTSRESPONSE']._serialized_end = 700
    _globals['_COMMANDMETADATASREQUEST']._serialized_start = 702
    _globals['_COMMANDMETADATASREQUEST']._serialized_end = 746
    _globals['_COMMANDMETADATASRESPONSE']._serialized_start = 748
    _globals['_COMMANDMETADATASRESPONSE']._serialized_end = 836
    _globals['_DEVICESTATUSREQUEST']._serialized_start = 838
    _globals['_DEVICESTATUSREQUEST']._serialized_end = 878
    _globals['_DEVICESTATESTREAMREQUEST']._serialized_start = 880
    _globals['_DEVICESTATESTREAMREQUEST']._serialized_end = 997
    _globals['_DEVICESTATEREQUEST']._serialized_start = 999
    _globals['_DEVICESTATEREQUEST']._serialized_end = 1038
    _globals['_DEVICESTATERESPONSE']._serialized_start = 1040
    _globals['_DEVICESTATERESPONSE']._serialized_end = 1104
    _globals['_DEVICESTATESCHEMAREQUEST']._serialized_start = 1106
    _globals['_DEVICESTATESCHEMAREQUEST']._serialized_end = 1151
    _globals['_DEVICESTATESCHEMARESPONSE']._serialized_start = 1153
    _globals['_DEVICESTATESCHEMARESPONSE']._serialized_end = 1230
    _globals['_DEVICELOGGERSETTINGSREQUEST']._serialized_start = 1232
    _globals['_DEVICELOGGERSETTINGSREQUEST']._serialized_end = 1280
    _globals['_DEVICELOGGERSRESPONSE']._serialized_start = 1282
    _globals['_DEVICELOGGERSRESPONSE']._serialized_end = 1368
    _globals['_DEVICEINFOREQUEST']._serialized_start = 1370
    _globals['_DEVICEINFOREQUEST']._serialized_end = 1408
    _globals['_DEVICESETTINGSREQUEST']._serialized_start = 1410
    _globals['_DEVICESETTINGSREQUEST']._serialized_end = 1452
    _globals['_DEVICEACTIVATEREQUEST']._serialized_start = 1454
    _globals['_DEVICEACTIVATEREQUEST']._serialized_end = 1496
    _globals['_DEVICECONFIGREQUEST']._serialized_start = 1498
    _globals['_DEVICECONFIGREQUEST']._serialized_end = 1540
    _globals['_DEVICECONFIGRESPONSE']._serialized_start = 1542
    _globals['_DEVICECONFIGRESPONSE']._serialized_end = 1618
    _globals['_REMOTEDEVICECONFIGRESPONSE']._serialized_start = 1620
    _globals['_REMOTEDEVICECONFIGRESPONSE']._serialized_end = 1708
    _globals['_ADDREMOTEDEVICEREQUEST']._serialized_start = 1710
    _globals['_ADDREMOTEDEVICEREQUEST']._serialized_end = 1761
    _globals['_ADDREMOTEDEVICERESPONSE']._serialized_start = 1763
    _globals['_ADDREMOTEDEVICERESPONSE']._serialized_end = 1870
    _globals['_UPDATEREMOTEDEVICEREQUEST']._serialized_start = 1872
    _globals['_UPDATEREMOTEDEVICEREQUEST']._serialized_end = 1972
    _globals['_UPDATEREMOTEDEVICERESPONSE']._serialized_start = 1974
    _globals['_UPDATEREMOTEDEVICERESPONSE']._serialized_end = 2065
    _globals['_REMOVEREMOTEDEVICEREQUEST']._serialized_start = 2067
    _globals['_REMOVEREMOTEDEVICEREQUEST']._serialized_end = 2113
    _globals['_REMOVEREMOTEDEVICERESPONSE']._serialized_start = 2115
    _globals['_REMOVEREMOTEDEVICERESPONSE']._serialized_end = 2206
    _globals['_ADDDEVICEREQUEST']._serialized_start = 2208
    _globals['_ADDDEVICEREQUEST']._serialized_end = 2328
    _globals['_ADDDEVICERESPONSE']._serialized_start = 2330
    _globals['_ADDDEVICERESPONSE']._serialized_end = 2389
    _globals['_AVAILABLEDEVICEDRIVERSRESPONSE']._serialized_start = 2391
    _globals['_AVAILABLEDEVICEDRIVERSRESPONSE']._serialized_end = 2447
    _globals['_GETDEVICECONFIGREQUEST']._serialized_start = 2449
    _globals['_GETDEVICECONFIGREQUEST']._serialized_end = 2513
    _globals['_AVAILABLEDEVICESRESPONSE']._serialized_start = 2515
    _globals['_AVAILABLEDEVICESRESPONSE']._serialized_end = 2604
    _globals['_UPDATEDEVICEREQUEST']._serialized_start = 2606
    _globals['_UPDATEDEVICEREQUEST']._serialized_end = 2707
    _globals['_UPDATEDEVICERESPONSE']._serialized_start = 2709
    _globals['_UPDATEDEVICERESPONSE']._serialized_end = 2794
    _globals['_REMOVEDEVICEREQUEST']._serialized_start = 2796
    _globals['_REMOVEDEVICEREQUEST']._serialized_end = 2836
    _globals['_REMOVEDEVICERESPONSE']._serialized_start = 2838
    _globals['_REMOVEDEVICERESPONSE']._serialized_end = 2923
    _globals['_ARESDEVICES']._serialized_start = 2926
    _globals['_ARESDEVICES']._serialized_end = 5764


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'request_metadata.proto')
_sym_db = _symbol_database.Default()
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16request_metadata.proto\x12\x0eares.datamodel\x1a\x1fgoogle/protobuf/timestamp.proto"\xa4\x01\n\x0fRequestMetadata\x12\x13\n\x0bsystem_name\x18\x01 \x01(\t\x12\x15\n\rcampaign_name\x18\x02 \x01(\t\x12\x13\n\x0bcampaign_id\x18\x03 \x01(\t\x12\x15\n\rexperiment_id\x18\x04 \x01(\t\x129\n\x15experiment_start_time\x18\x05 \x01(\x0b2\x1a.google.protobuf.Timestampb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'request_metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_REQUESTMETADATA']._serialized_start = 76
    _globals['_REQUESTMETADATA']._serialized_end = 240


from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Limits(_message.Message):
    __slots__ = ("minimum", "maximum")
    MINIMUM_FIELD_NUMBER: _ClassVar[int]
    MAXIMUM_FIELD_NUMBER: _ClassVar[int]
    minimum: float
    maximum: float
    def __init__(self, minimum: _Optional[float] = ..., maximum: _Optional[float] = ...) -> None: ...



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'ares_quantity_type.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18ares_quantity_type.proto\x12\x0eares.datamodel*\xca\x06\n\x0cQuantityType\x12\x1d\n\x19QUANTITY_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06LENGTH\x10\x01\x12\x08\n\x04AREA\x10\x02\x12\n\n\x06VOLUME\x10\x03\x12\x0c\n\x08DURATION\x10\x04\x12\t\n\x05SPEED\x10\x05\x12\x10\n\x0cACCELERATION\x10\x06\x12\r\n\tFREQUENCY\x10\x07\x12\x08\n\x04MASS\x10\x08\x12\x17\n\x13AMOUNT_OF_SUBSTANCE\x10\t\x12\x0e\n\nMOLAR_MASS\x10\n\x12\x0f\n\x0bTEMPERATURE\x10\x0b\x12\x15\n\x11TEMPERATURE_DELTA\x10\x0c\x12\x0c\n\x08PRESSURE\x10\r\x12\x0c\n\x08MOLARITY\x10\x0e\x12\x16\n\x12MASS_CONCENTRATION\x10\x0f\x12\x18\n\x14VOLUME_CONCENTRATION\x10\x10\x12\x11\n\rMASS_FRACTION\x10\x11\x12\x0b\n\x07DENSITY\x10\x12\x12\x15\n\x11DYNAMIC_VISCOSITY\x10\x13\x12\x17\n\x13KINEMATIC_VISCOSITY\x10\x14\x12\x0f\n\x0bVOLUME_FLOW\x10\x15\x12\r\n\tMASS_FLOW\x10\x16\x12\r\n\tMASS_FLUX\x10\x17\x12\x1b\n\x17ROTATIONAL_ACCELERATION\x10\x18\x12\x14\n\x10ROTATIONAL_SPEED\x10\x19\x12\n\n\x06ENERGY\x10\x1a\x12\t\n\x05POWER\x10\x1b\x12\r\n\tHEAT_FLUX\x10\x1c\x12\x13\n\x0fSPECIFIC_ENERGY\x10\x1d\x12\x18\n\x14THERMAL_CONDUCTIVITY\x10\x1e\x12\t\n\x05FORCE\x10\x1f\x12\x14\n\x10FORCE_PER_LENGTH\x10 \x12\t\n\x05RATIO\x10!\x12\x0e\n\nIRRADIANCE\x10"\x12\x0f\n\x0bILLUMINANCE\x10#\x12\x18\n\x14TEMPERATURE_GRADIENT\x10$\x12\x16\n\x12ELECTRIC_POTENTIAL\x10%\x12\x14\n\x10ELECTRIC_CURRENT\x10&\x12\x17\n\x13ELECTRIC_RESISTANCE\x10\'\x12\x19\n\x15ELECTRIC_CONDUCTIVITY\x10(\x12\x18\n\x14ELECTRIC_CONDUCTANCE\x10)\x12\x15\n\x11RATIO_CHANGE_RATE\x10*\x12\x15\n\x11RELATIVE_HUMIDITY\x10+b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ares_quantity_type_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_QUANTITYTYPE']._serialized_start = 45
    _globals['_QUANTITYTYPE']._serialized_end = 887


import ares_struct_pb2 as _ares_struct_pb2
from device import device_connection_info_pb2 as _device_connection_info_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DeviceConfig(_message.Message):
    __slots__ = ("unique_id", "device_name", "device_id", "driver_id", "is_simulated", "serial_info", "device_settings")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    DEVICE_NAME_FIELD_NUMBER: _ClassVar[int]
    DEVICE_ID_FIELD_NUMBER: _ClassVar[int]
    DRIVER_ID_FIELD_NUMBER: _ClassVar[int]
    IS_SIMULATED_FIELD_NUMBER: _ClassVar[int]
    SERIAL_INFO_FIELD_NUMBER: _ClassVar[int]
    DEVICE_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    device_name: str
    device_id: str
    driver_id: str
    is_simulated: bool
    serial_info: _device_connection_info_pb2.SerialConnection
    device_settings: _ares_struct_pb2.AresStruct
    def __init__(self, unique_id: _Optional[str] = ..., device_name: _Optional[str] = ..., device_id: _Optional[str] = ..., driver_id: _Optional[str] = ..., is_simulated: bool = ..., serial_info: _Optional[_Union[_device_connection_info_pb2.SerialConnection, _Mapping]] = ..., device_settings: _Optional[_Union[_ares_struct_pb2.AresStruct, _Mapping]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class AresDataType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNSPECIFIED_TYPE: _ClassVar[AresDataType]
    NULL: _ClassVar[AresDataType]
    BOOLEAN: _ClassVar[AresDataType]
    STRING: _ClassVar[AresDataType]
    NUMBER: _ClassVar[AresDataType]
    STRING_ARRAY: _ClassVar[AresDataType]
    NUMBER_ARRAY: _ClassVar[AresDataType]
    LIST: _ClassVar[AresDataType]
    STRUCT: _ClassVar[AresDataType]
    BYTE_ARRAY: _ClassVar[AresDataType]
    ANY: _ClassVar[AresDataType]
    UNIT: _ClassVar[AresDataType]
    FUNCTION: _ClassVar[AresDataType]
    QUANTITY: _ClassVar[AresDataType]
UNSPECIFIED_TYPE: AresDataType
NULL: AresDataType
BOOLEAN: AresDataType
STRING: AresDataType
NUMBER: AresDataType
STRING_ARRAY: AresDataType
NUMBER_ARRAY: AresDataType
LIST: AresDataType
STRUCT: AresDataType
BYTE_ARRAY: AresDataType
ANY: AresDataType
UNIT: AresDataType
FUNCTION: AresDataType
QUANTITY: AresDataType



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'templates/parameter_metadata.proto')
_sym_db = _symbol_database.Default()
from .. import limits_pb2 as limits__pb2
from .. import ares_data_schema_pb2 as ares__data__schema__pb2
from .. import ares_struct_pb2 as ares__struct__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n"templates/parameter_metadata.proto\x12\x18ares.datamodel.templates\x1a\x0climits.proto\x1a\x16ares_data_schema.proto\x1a\x11ares_struct.proto"\xa4\x03\n\x11ParameterMetadata\x12\x16\n\tunique_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04unit\x18\x03 \x01(\t\x12+\n\x0bconstraints\x18\x04 \x03(\x0b2\x16.ares.datamodel.Limits\x12\r\n\x05index\x18\x05 \x01(\x03\x12\x18\n\x0boutput_name\x18\x06 \x01(\tH\x01\x88\x01\x01\x12\x15\n\rnot_plannable\x18\x07 \x01(\x08\x12\x13\n\x0buse_default\x18\x08 \x01(\x08\x124\n\x06schema\x18\t \x01(\x0b2\x1f.ares.datamodel.AresValueSchemaH\x02\x88\x01\x01\x12\x14\n\x0cplanner_name\x18\n \x01(\t\x12\x1b\n\x13planner_description\x18\x0b \x01(\t\x125\n\rinitial_value\x18\x0c \x01(\x0b2\x19.ares.datamodel.AresValueH\x03\x88\x01\x01B\x0c\n\n_unique_idB\x0e\n\x0c_output_nameB\t\n\x07_schemaB\x10\n\x0e_initial_valueb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'templates.parameter_metadata_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_PARAMETERMETADATA']._serialized_start = 122
    _globals['_PARAMETERMETADATA']._serialized_end = 542


from .quantity_units import convert_quantity_value_unit
from .quantity_units import from_pint_quantity
from .quantity_units import to_quantity_schema
from .quantity_units import get_unit_registry
from .quantity_units import to_pint_quantity
from .quantity_validation import validate_quantity_type_match
from .quantity_validation import validate_quantity_value

try:
    from .ares_quantity_type_pb2 import QuantityType
except Exception: 
    QuantityType = None

__all__ = [
    "convert_quantity_value_unit",
    "from_pint_quantity",
    "to_quantity_schema"
    "get_unit_registry",
    "to_pint_quantity",
    "validate_quantity_type_match",
    "validate_quantity_value",
]

if QuantityType is not None:
    __all__.append("QuantityType")



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class AresCampaignTag(_message.Message):
    __slots__ = ("unique_id", "tag_name")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    TAG_NAME_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    tag_name: str
    def __init__(self, unique_id: _Optional[str] = ..., tag_name: _Optional[str] = ...) -> None: ...



from templates import experiment_template_pb2 as _experiment_template_pb2
from templates import parameter_pb2 as _parameter_pb2
import ares_struct_pb2 as _ares_struct_pb2
import analysis_overview_pb2 as _analysis_overview_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ExperimentOverview(_message.Message):
    __slots__ = ("unique_id", "template", "result", "parameters", "analysis_overview")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    ANALYSIS_OVERVIEW_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    template: _experiment_template_pb2.ExperimentTemplate
    result: _ares_struct_pb2.AresStruct
    parameters: _containers.RepeatedCompositeFieldContainer[_parameter_pb2.Parameter]
    analysis_overview: _analysis_overview_pb2.AnalysisOverview
    def __init__(self, unique_id: _Optional[str] = ..., template: _Optional[_Union[_experiment_template_pb2.ExperimentTemplate, _Mapping]] = ..., result: _Optional[_Union[_ares_struct_pb2.AresStruct, _Mapping]] = ..., parameters: _Optional[_Iterable[_Union[_parameter_pb2.Parameter, _Mapping]]] = ..., analysis_overview: _Optional[_Union[_analysis_overview_pb2.AnalysisOverview, _Mapping]] = ...) -> None: ...



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ScriptRange(_message.Message):
    __slots__ = ("start_line", "start_column", "end_line", "end_column")
    START_LINE_FIELD_NUMBER: _ClassVar[int]
    START_COLUMN_FIELD_NUMBER: _ClassVar[int]
    END_LINE_FIELD_NUMBER: _ClassVar[int]
    END_COLUMN_FIELD_NUMBER: _ClassVar[int]
    start_line: int
    start_column: int
    end_line: int
    end_column: int
    def __init__(self, start_line: _Optional[int] = ..., start_column: _Optional[int] = ..., end_line: _Optional[int] = ..., end_column: _Optional[int] = ...) -> None: ...

class SymbolLocation(_message.Message):
    __slots__ = ("identifier", "parent_identifier", "range")
    IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    PARENT_IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    RANGE_FIELD_NUMBER: _ClassVar[int]
    identifier: str
    parent_identifier: str
    range: ScriptRange
    def __init__(self, identifier: _Optional[str] = ..., parent_identifier: _Optional[str] = ..., range: _Optional[_Union[ScriptRange, _Mapping]] = ...) -> None: ...



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'device/device_description.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fdevice/device_description.proto\x12\x15ares.datamodel.device"?\n\x15AresDeviceDescription\x12\x13\n\x0bdevice_name\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\tb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'device.device_description_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_ARESDEVICEDESCRIPTION']._serialized_start = 58
    _globals['_ARESDEVICEDESCRIPTION']._serialized_end = 121


"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'scripting/function_invocation_kind.proto')
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(scripting/function_invocation_kind.proto\x12\x18ares.datamodel.scripting*\xd7\x01\n\x16FunctionInvocationKind\x12(\n$FUNCTION_INVOCATION_KIND_UNSPECIFIED\x10\x00\x12#\n\x1fFUNCTION_INVOCATION_KIND_SYSTEM\x10\x01\x12&\n"FUNCTION_INVOCATION_KIND_EXTENSION\x10\x02\x12!\n\x1dFUNCTION_INVOCATION_KIND_USER\x10\x03\x12#\n\x1fFUNCTION_INVOCATION_KIND_LAMBDA\x10\x04b\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'scripting.function_invocation_kind_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_FUNCTIONINVOCATIONKIND']._serialized_start = 71
    _globals['_FUNCTIONINVOCATIONKIND']._serialized_end = 286


"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings
from .analyzing import analysis_pb2 as analyzing_dot_analysis__pb2
from . import ares_analysis_service_pb2 as ares__analysis__service__pb2
from . import validation_result_pb2 as validation__result__pb2
GRPC_GENERATED_VERSION = '1.78.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False
try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True
if _version_not_supported:
    raise RuntimeError(f'The grpc package installed is at version {GRPC_VERSION},' + ' but the generated code in ares_analysis_service_pb2_grpc.py depends on' + f' grpcio>={GRPC_GENERATED_VERSION}.' + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}' + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.')

class AresAnalysisServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ValidateInputs = channel.unary_unary('/ares.services.AresAnalysisService/ValidateInputs', request_serializer=ares__analysis__service__pb2.InputValidationRequest.SerializeToString, response_deserializer=validation__result__pb2.ValidationResult.FromString, _registered_method=True)
        self.Analyze = channel.unary_unary('/ares.services.AresAnalysisService/Analyze', request_serializer=ares__analysis__service__pb2.AnalysisRequest.SerializeToString, response_deserializer=analyzing_dot_analysis__pb2.Analysis.FromString, _registered_method=True)
        self.GetAnalyzerParameters = channel.unary_unary('/ares.services.AresAnalysisService/GetAnalyzerParameters', request_serializer=ares__analysis__service__pb2.AnalyzerParametersRequest.SerializeToString, response_deserializer=ares__analysis__service__pb2.AnalyzerParametersResponse.FromString, _registered_method=True)

class AresAnalysisServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ValidateInputs(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Analyze(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnalyzerParameters(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

def add_AresAnalysisServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {'ValidateInputs': grpc.unary_unary_rpc_method_handler(servicer.ValidateInputs, request_deserializer=ares__analysis__service__pb2.InputValidationRequest.FromString, response_serializer=validation__result__pb2.ValidationResult.SerializeToString), 'Analyze': grpc.unary_unary_rpc_method_handler(servicer.Analyze, request_deserializer=ares__analysis__service__pb2.AnalysisRequest.FromString, response_serializer=analyzing_dot_analysis__pb2.Analysis.SerializeToString), 'GetAnalyzerParameters': grpc.unary_unary_rpc_method_handler(servicer.GetAnalyzerParameters, request_deserializer=ares__analysis__service__pb2.AnalyzerParametersRequest.FromString, response_serializer=ares__analysis__service__pb2.AnalyzerParametersResponse.SerializeToString)}
    generic_handler = grpc.method_handlers_generic_handler('ares.services.AresAnalysisService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('ares.services.AresAnalysisService', rpc_method_handlers)

class AresAnalysisService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ValidateInputs(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalysisService/ValidateInputs', ares__analysis__service__pb2.InputValidationRequest.SerializeToString, validation__result__pb2.ValidationResult.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def Analyze(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalysisService/Analyze', ares__analysis__service__pb2.AnalysisRequest.SerializeToString, analyzing_dot_analysis__pb2.Analysis.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def GetAnalyzerParameters(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalysisService/GetAnalyzerParameters', ares__analysis__service__pb2.AnalyzerParametersRequest.SerializeToString, ares__analysis__service__pb2.AnalyzerParametersResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)


from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Outcome(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNSPECIFIED_OUTCOME: _ClassVar[Outcome]
    SUCCESS: _ClassVar[Outcome]
    FAILURE: _ClassVar[Outcome]
    WARNING: _ClassVar[Outcome]
    CANCELED: _ClassVar[Outcome]
UNSPECIFIED_OUTCOME: Outcome
SUCCESS: Outcome
FAILURE: Outcome
WARNING: Outcome
CANCELED: Outcome



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'ares_validation.proto')
_sym_db = _symbol_database.Default()
from .templates import campaign_template_pb2 as templates_dot_campaign__template__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from .templates import experiment_template_pb2 as templates_dot_experiment__template__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15ares_validation.proto\x12\rares.services\x1a!templates/campaign_template.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a#templates/experiment_template.proto"{\n\x19AnalyzerValidationRequest\x12\x13\n\x0banalyzer_id\x18\x01 \x01(\t\x12I\n\x13experiment_template\x18\x02 \x01(\x0b2,.ares.datamodel.templates.ExperimentTemplate"7\n\x12ValidationResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x10\n\x08messages\x18\x02 \x03(\t2\xb9\x02\n\x0eAresValidation\x12e\n\x14ValidateFullCampaign\x12*.ares.datamodel.templates.CampaignTemplate\x1a!.ares.services.ValidationResponse\x12h\n\x19ValidateAnalyzerSelection\x12(.ares.services.AnalyzerValidationRequest\x1a!.ares.services.ValidationResponse\x12V\n\x19ValidateRegisteredDevices\x12\x16.google.protobuf.Empty\x1a!.ares.services.ValidationResponseb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ares_validation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_ANALYZERVALIDATIONREQUEST']._serialized_start = 141
    _globals['_ANALYZERVALIDATIONREQUEST']._serialized_end = 264
    _globals['_VALIDATIONRESPONSE']._serialized_start = 266
    _globals['_VALIDATIONRESPONSE']._serialized_end = 321
    _globals['_ARESVALIDATION']._serialized_start = 324
    _globals['_ARESVALIDATION']._serialized_end = 637


"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings
from .analyzing import analyzer_settings_pb2 as analyzing_dot_analyzer__settings__pb2
from . import ares_analyzer_management_service_pb2 as ares__analyzer__management__service__pb2
from . import ares_struct_pb2 as ares__struct__pb2
from .connection import connection_state_pb2 as connection_dot_connection__state__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
GRPC_GENERATED_VERSION = '1.78.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False
try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True
if _version_not_supported:
    raise RuntimeError(f'The grpc package installed is at version {GRPC_VERSION},' + ' but the generated code in ares_analyzer_management_service_pb2_grpc.py depends on' + f' grpcio>={GRPC_GENERATED_VERSION}.' + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}' + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.')

class AresAnalyzerManagementServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetState = channel.unary_unary('/ares.services.AresAnalyzerManagementService/GetState', request_serializer=connection_dot_connection__state__pb2.StateRequest.SerializeToString, response_deserializer=connection_dot_connection__state__pb2.StateResponse.FromString, _registered_method=True)
        self.GetInfo = channel.unary_unary('/ares.services.AresAnalyzerManagementService/GetInfo', request_serializer=ares__analyzer__management__service__pb2.AnalyzerInfoRequest.SerializeToString, response_deserializer=ares__analyzer__management__service__pb2.AnalyzerInfoResponse.FromString, _registered_method=True)
        self.SetAnalyzerSettings = channel.unary_unary('/ares.services.AresAnalyzerManagementService/SetAnalyzerSettings', request_serializer=analyzing_dot_analyzer__settings__pb2.AnalyzerSettings.SerializeToString, response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString, _registered_method=True)
        self.GetAnalyzerSettings = channel.unary_unary('/ares.services.AresAnalyzerManagementService/GetAnalyzerSettings', request_serializer=ares__analyzer__management__service__pb2.AnalyzerSettingsRequest.SerializeToString, response_deserializer=ares__struct__pb2.AresStruct.FromString, _registered_method=True)
        self.GetAllAnalyzers = channel.unary_unary('/ares.services.AresAnalyzerManagementService/GetAllAnalyzers', request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString, response_deserializer=ares__analyzer__management__service__pb2.GetAllAnalyzersResponse.FromString, _registered_method=True)
        self.AddRemoteAnalyzer = channel.unary_unary('/ares.services.AresAnalyzerManagementService/AddRemoteAnalyzer', request_serializer=ares__analyzer__management__service__pb2.AddRemoteAnalyzerRequest.SerializeToString, response_deserializer=ares__analyzer__management__service__pb2.AddRemoteAnalyzerResponse.FromString, _registered_method=True)
        self.UpdateRemoteAnalyzer = channel.unary_unary('/ares.services.AresAnalyzerManagementService/UpdateRemoteAnalyzer', request_serializer=ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerRequest.SerializeToString, response_deserializer=ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerResponse.FromString, _registered_method=True)
        self.RemoveRemoteAnalyzer = channel.unary_unary('/ares.services.AresAnalyzerManagementService/RemoveRemoteAnalyzer', request_serializer=ares__analyzer__management__service__pb2.RemoveRemoteAnalyzerRequest.SerializeToString, response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString, _registered_method=True)

class AresAnalyzerManagementServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAnalyzerSettings(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAnalyzerSettings(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllAnalyzers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddRemoteAnalyzer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateRemoteAnalyzer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveRemoteAnalyzer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

def add_AresAnalyzerManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {'GetState': grpc.unary_unary_rpc_method_handler(servicer.GetState, request_deserializer=connection_dot_connection__state__pb2.StateRequest.FromString, response_serializer=connection_dot_connection__state__pb2.StateResponse.SerializeToString), 'GetInfo': grpc.unary_unary_rpc_method_handler(servicer.GetInfo, request_deserializer=ares__analyzer__management__service__pb2.AnalyzerInfoRequest.FromString, response_serializer=ares__analyzer__management__service__pb2.AnalyzerInfoResponse.SerializeToString), 'SetAnalyzerSettings': grpc.unary_unary_rpc_method_handler(servicer.SetAnalyzerSettings, request_deserializer=analyzing_dot_analyzer__settings__pb2.AnalyzerSettings.FromString, response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString), 'GetAnalyzerSettings': grpc.unary_unary_rpc_method_handler(servicer.GetAnalyzerSettings, request_deserializer=ares__analyzer__management__service__pb2.AnalyzerSettingsRequest.FromString, response_serializer=ares__struct__pb2.AresStruct.SerializeToString), 'GetAllAnalyzers': grpc.unary_unary_rpc_method_handler(servicer.GetAllAnalyzers, request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString, response_serializer=ares__analyzer__management__service__pb2.GetAllAnalyzersResponse.SerializeToString), 'AddRemoteAnalyzer': grpc.unary_unary_rpc_method_handler(servicer.AddRemoteAnalyzer, request_deserializer=ares__analyzer__management__service__pb2.AddRemoteAnalyzerRequest.FromString, response_serializer=ares__analyzer__management__service__pb2.AddRemoteAnalyzerResponse.SerializeToString), 'UpdateRemoteAnalyzer': grpc.unary_unary_rpc_method_handler(servicer.UpdateRemoteAnalyzer, request_deserializer=ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerRequest.FromString, response_serializer=ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerResponse.SerializeToString), 'RemoveRemoteAnalyzer': grpc.unary_unary_rpc_method_handler(servicer.RemoveRemoteAnalyzer, request_deserializer=ares__analyzer__management__service__pb2.RemoveRemoteAnalyzerRequest.FromString, response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString)}
    generic_handler = grpc.method_handlers_generic_handler('ares.services.AresAnalyzerManagementService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('ares.services.AresAnalyzerManagementService', rpc_method_handlers)

class AresAnalyzerManagementService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetState(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/GetState', connection_dot_connection__state__pb2.StateRequest.SerializeToString, connection_dot_connection__state__pb2.StateResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def GetInfo(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/GetInfo', ares__analyzer__management__service__pb2.AnalyzerInfoRequest.SerializeToString, ares__analyzer__management__service__pb2.AnalyzerInfoResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def SetAnalyzerSettings(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/SetAnalyzerSettings', analyzing_dot_analyzer__settings__pb2.AnalyzerSettings.SerializeToString, google_dot_protobuf_dot_empty__pb2.Empty.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def GetAnalyzerSettings(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/GetAnalyzerSettings', ares__analyzer__management__service__pb2.AnalyzerSettingsRequest.SerializeToString, ares__struct__pb2.AresStruct.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def GetAllAnalyzers(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/GetAllAnalyzers', google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString, ares__analyzer__management__service__pb2.GetAllAnalyzersResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def AddRemoteAnalyzer(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/AddRemoteAnalyzer', ares__analyzer__management__service__pb2.AddRemoteAnalyzerRequest.SerializeToString, ares__analyzer__management__service__pb2.AddRemoteAnalyzerResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def UpdateRemoteAnalyzer(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/UpdateRemoteAnalyzer', ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerRequest.SerializeToString, ares__analyzer__management__service__pb2.UpdateRemoteAnalyzerResponse.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)

    @staticmethod
    def RemoveRemoteAnalyzer(request, target, options=(), channel_credentials=None, call_credentials=None, insecure=False, compression=None, wait_for_ready=None, timeout=None, metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ares.services.AresAnalyzerManagementService/RemoveRemoteAnalyzer', ares__analyzer__management__service__pb2.RemoveRemoteAnalyzerRequest.SerializeToString, google_dot_protobuf_dot_empty__pb2.Empty.FromString, options, channel_credentials, insecure, call_credentials, compression, wait_for_ready, timeout, metadata, _registered_method=True)


import datetime

from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from templates import campaign_template_pb2 as _campaign_template_pb2
import project_pb2 as _project_pb2
import execution_status_messages_pb2 as _execution_status_messages_pb2
import execution_summary_messages_pb2 as _execution_summary_messages_pb2
import ares_campaign_tag_pb2 as _ares_campaign_tag_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TagRequest(_message.Message):
    __slots__ = ("tag",)
    TAG_FIELD_NUMBER: _ClassVar[int]
    tag: _ares_campaign_tag_pb2.AresCampaignTag
    def __init__(self, tag: _Optional[_Union[_ares_campaign_tag_pb2.AresCampaignTag, _Mapping]] = ...) -> None: ...

class TagsResponse(_message.Message):
    __slots__ = ("available_tags",)
    AVAILABLE_TAGS_FIELD_NUMBER: _ClassVar[int]
    available_tags: _containers.RepeatedCompositeFieldContainer[_ares_campaign_tag_pb2.AresCampaignTag]
    def __init__(self, available_tags: _Optional[_Iterable[_Union[_ares_campaign_tag_pb2.AresCampaignTag, _Mapping]]] = ...) -> None: ...

class AddOrUpdateCampaignRequest(_message.Message):
    __slots__ = ("template",)
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    template: _campaign_template_pb2.CampaignTemplate
    def __init__(self, template: _Optional[_Union[_campaign_template_pb2.CampaignTemplate, _Mapping]] = ...) -> None: ...

class RequestById(_message.Message):
    __slots__ = ("unique_id",)
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    def __init__(self, unique_id: _Optional[str] = ...) -> None: ...

class CampaignRequest(_message.Message):
    __slots__ = ("campaign_name", "unique_id")
    CAMPAIGN_NAME_FIELD_NUMBER: _ClassVar[int]
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    campaign_name: str
    unique_id: str
    def __init__(self, campaign_name: _Optional[str] = ..., unique_id: _Optional[str] = ...) -> None: ...

class GetAllCampaignsRequest(_message.Message):
    __slots__ = ("file_path",)
    FILE_PATH_FIELD_NUMBER: _ClassVar[int]
    file_path: str
    def __init__(self, file_path: _Optional[str] = ...) -> None: ...

class GetAllCampaignsResponse(_message.Message):
    __slots__ = ("campaigns",)
    CAMPAIGNS_FIELD_NUMBER: _ClassVar[int]
    campaigns: _containers.RepeatedCompositeFieldContainer[CampaignTemplateSummary]
    def __init__(self, campaigns: _Optional[_Iterable[_Union[CampaignTemplateSummary, _Mapping]]] = ...) -> None: ...

class CampaignTemplateSummary(_message.Message):
    __slots__ = ("unique_id", "campaign_name")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_NAME_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    campaign_name: str
    def __init__(self, unique_id: _Optional[str] = ..., campaign_name: _Optional[str] = ...) -> None: ...

class CampaignResponse(_message.Message):
    __slots__ = ("has_value", "value")
    HAS_VALUE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    has_value: bool
    value: _campaign_template_pb2.CampaignTemplate
    def __init__(self, has_value: bool = ..., value: _Optional[_Union[_campaign_template_pb2.CampaignTemplate, _Mapping]] = ...) -> None: ...

class StartCampaignRequest(_message.Message):
    __slots__ = ("user_notes", "campaign_tags")
    USER_NOTES_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_TAGS_FIELD_NUMBER: _ClassVar[int]
    user_notes: str
    campaign_tags: _containers.RepeatedCompositeFieldContainer[_ares_campaign_tag_pb2.AresCampaignTag]
    def __init__(self, user_notes: _Optional[str] = ..., campaign_tags: _Optional[_Iterable[_Union[_ares_campaign_tag_pb2.AresCampaignTag, _Mapping]]] = ...) -> None: ...

class ProjectRequest(_message.Message):
    __slots__ = ("project_name",)
    PROJECT_NAME_FIELD_NUMBER: _ClassVar[int]
    project_name: str
    def __init__(self, project_name: _Optional[str] = ...) -> None: ...

class ProjectsResponse(_message.Message):
    __slots__ = ("projects",)
    PROJECTS_FIELD_NUMBER: _ClassVar[int]
    projects: _containers.RepeatedCompositeFieldContainer[_project_pb2.Project]
    def __init__(self, projects: _Optional[_Iterable[_Union[_project_pb2.Project, _Mapping]]] = ...) -> None: ...

class CampaignExecutionStatusResponse(_message.Message):
    __slots__ = ("status",)
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: _execution_status_messages_pb2.CampaignExecutionStatus
    def __init__(self, status: _Optional[_Union[_execution_status_messages_pb2.CampaignExecutionStatus, _Mapping]] = ...) -> None: ...

class StartStopConditionsResponse(_message.Message):
    __slots__ = ("start_stop_conditions",)
    START_STOP_CONDITIONS_FIELD_NUMBER: _ClassVar[int]
    start_stop_conditions: _containers.RepeatedCompositeFieldContainer[StartStopCondition]
    def __init__(self, start_stop_conditions: _Optional[_Iterable[_Union[StartStopCondition, _Mapping]]] = ...) -> None: ...

class GetReplanRateResponse(_message.Message):
    __slots__ = ("ReplanRate",)
    REPLANRATE_FIELD_NUMBER: _ClassVar[int]
    ReplanRate: int
    def __init__(self, ReplanRate: _Optional[int] = ...) -> None: ...

class StartStopCondition(_message.Message):
    __slots__ = ("name", "message")
    NAME_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    name: str
    message: str
    def __init__(self, name: _Optional[str] = ..., message: _Optional[str] = ...) -> None: ...

class ReplanRate(_message.Message):
    __slots__ = ("ReplanRate",)
    REPLANRATE_FIELD_NUMBER: _ClassVar[int]
    ReplanRate: int
    def __init__(self, ReplanRate: _Optional[int] = ...) -> None: ...

class NumExperimentsCondition(_message.Message):
    __slots__ = ("num_experiments",)
    NUM_EXPERIMENTS_FIELD_NUMBER: _ClassVar[int]
    num_experiments: int
    def __init__(self, num_experiments: _Optional[int] = ...) -> None: ...

class ExperimentStopConditionResponse(_message.Message):
    __slots__ = ("active_condition", "description")
    ACTIVE_CONDITION_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    active_condition: str
    description: str
    def __init__(self, active_condition: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...

class AnalysisResultCondition(_message.Message):
    __slots__ = ("desired_result", "leeway")
    DESIRED_RESULT_FIELD_NUMBER: _ClassVar[int]
    LEEWAY_FIELD_NUMBER: _ClassVar[int]
    desired_result: float
    leeway: float
    def __init__(self, desired_result: _Optional[float] = ..., leeway: _Optional[float] = ...) -> None: ...

class AvailableCampaignExecutionSummariesResponse(_message.Message):
    __slots__ = ("available_campaign_summaries",)
    AVAILABLE_CAMPAIGN_SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    available_campaign_summaries: _containers.RepeatedCompositeFieldContainer[CampaignExecutionSummaryMetadata]
    def __init__(self, available_campaign_summaries: _Optional[_Iterable[_Union[CampaignExecutionSummaryMetadata, _Mapping]]] = ...) -> None: ...

class CampaignExecutionSummaryMetadata(_message.Message):
    __slots__ = ("campaign_name", "completion_time", "summary_id", "num_experiments")
    CAMPAIGN_NAME_FIELD_NUMBER: _ClassVar[int]
    COMPLETION_TIME_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_ID_FIELD_NUMBER: _ClassVar[int]
    NUM_EXPERIMENTS_FIELD_NUMBER: _ClassVar[int]
    campaign_name: str
    completion_time: _timestamp_pb2.Timestamp
    summary_id: str
    num_experiments: int
    def __init__(self, campaign_name: _Optional[str] = ..., completion_time: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., summary_id: _Optional[str] = ..., num_experiments: _Optional[int] = ...) -> None: ...

class CampaignExecutionSummaryRequest(_message.Message):
    __slots__ = ("summary_id",)
    SUMMARY_ID_FIELD_NUMBER: _ClassVar[int]
    summary_id: str
    def __init__(self, summary_id: _Optional[str] = ...) -> None: ...

class CheckExecutionEligibilityResponse(_message.Message):
    __slots__ = ("is_eligible", "error")
    IS_ELIGIBLE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    is_eligible: bool
    error: str
    def __init__(self, is_eligible: bool = ..., error: _Optional[str] = ...) -> None: ...

class GetCopyOfCampaignResponse(_message.Message):
    __slots__ = ("template", "serialized_json_data")
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    SERIALIZED_JSON_DATA_FIELD_NUMBER: _ClassVar[int]
    template: _campaign_template_pb2.CampaignTemplate
    serialized_json_data: str
    def __init__(self, template: _Optional[_Union[_campaign_template_pb2.CampaignTemplate, _Mapping]] = ..., serialized_json_data: _Optional[str] = ...) -> None: ...



import datetime

from google.protobuf import timestamp_pb2 as _timestamp_pb2
import experiment_overview_pb2 as _experiment_overview_pb2
import command_result_pb2 as _command_result_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ExecutionInfo(_message.Message):
    __slots__ = ("unique_id", "time_started", "time_finished", "timezone", "localtime_offset")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    TIME_STARTED_FIELD_NUMBER: _ClassVar[int]
    TIME_FINISHED_FIELD_NUMBER: _ClassVar[int]
    TIMEZONE_FIELD_NUMBER: _ClassVar[int]
    LOCALTIME_OFFSET_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    time_started: _timestamp_pb2.Timestamp
    time_finished: _timestamp_pb2.Timestamp
    timezone: str
    localtime_offset: str
    def __init__(self, unique_id: _Optional[str] = ..., time_started: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., time_finished: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., timezone: _Optional[str] = ..., localtime_offset: _Optional[str] = ...) -> None: ...

class CampaignExecutionSummary(_message.Message):
    __slots__ = ("unique_id", "campaign_id", "experiment_summaries", "execution_info", "campaign_name", "campaign_tags", "campaign_notes", "startup_execution_summary", "closeout_execution_summary")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_ID_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_INFO_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_NAME_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_TAGS_FIELD_NUMBER: _ClassVar[int]
    CAMPAIGN_NOTES_FIELD_NUMBER: _ClassVar[int]
    STARTUP_EXECUTION_SUMMARY_FIELD_NUMBER: _ClassVar[int]
    CLOSEOUT_EXECUTION_SUMMARY_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    campaign_id: str
    experiment_summaries: _containers.RepeatedCompositeFieldContainer[ExperimentExecutionSummary]
    execution_info: ExecutionInfo
    campaign_name: str
    campaign_tags: str
    campaign_notes: str
    startup_execution_summary: ExperimentExecutionSummary
    closeout_execution_summary: ExperimentExecutionSummary
    def __init__(self, unique_id: _Optional[str] = ..., campaign_id: _Optional[str] = ..., experiment_summaries: _Optional[_Iterable[_Union[ExperimentExecutionSummary, _Mapping]]] = ..., execution_info: _Optional[_Union[ExecutionInfo, _Mapping]] = ..., campaign_name: _Optional[str] = ..., campaign_tags: _Optional[str] = ..., campaign_notes: _Optional[str] = ..., startup_execution_summary: _Optional[_Union[ExperimentExecutionSummary, _Mapping]] = ..., closeout_execution_summary: _Optional[_Union[ExperimentExecutionSummary, _Mapping]] = ...) -> None: ...

class ExperimentExecutionSummary(_message.Message):
    __slots__ = ("unique_id", "experiment_id", "step_summaries", "execution_info", "experiment_overview", "result_output_path")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_ID_FIELD_NUMBER: _ClassVar[int]
    STEP_SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_INFO_FIELD_NUMBER: _ClassVar[int]
    EXPERIMENT_OVERVIEW_FIELD_NUMBER: _ClassVar[int]
    RESULT_OUTPUT_PATH_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    experiment_id: str
    step_summaries: _containers.RepeatedCompositeFieldContainer[StepExecutionSummary]
    execution_info: ExecutionInfo
    experiment_overview: _experiment_overview_pb2.ExperimentOverview
    result_output_path: str
    def __init__(self, unique_id: _Optional[str] = ..., experiment_id: _Optional[str] = ..., step_summaries: _Optional[_Iterable[_Union[StepExecutionSummary, _Mapping]]] = ..., execution_info: _Optional[_Union[ExecutionInfo, _Mapping]] = ..., experiment_overview: _Optional[_Union[_experiment_overview_pb2.ExperimentOverview, _Mapping]] = ..., result_output_path: _Optional[str] = ...) -> None: ...

class StepExecutionSummary(_message.Message):
    __slots__ = ("unique_id", "step_id", "command_summaries", "execution_info")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    STEP_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_SUMMARIES_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_INFO_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    step_id: str
    command_summaries: _containers.RepeatedCompositeFieldContainer[CommandExecutionSummary]
    execution_info: ExecutionInfo
    def __init__(self, unique_id: _Optional[str] = ..., step_id: _Optional[str] = ..., command_summaries: _Optional[_Iterable[_Union[CommandExecutionSummary, _Mapping]]] = ..., execution_info: _Optional[_Union[ExecutionInfo, _Mapping]] = ...) -> None: ...

class CommandExecutionSummary(_message.Message):
    __slots__ = ("unique_id", "command_id", "execution_info", "result", "command_name", "command_description", "template_id")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_ID_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_INFO_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    COMMAND_NAME_FIELD_NUMBER: _ClassVar[int]
    COMMAND_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    TEMPLATE_ID_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    command_id: str
    execution_info: ExecutionInfo
    result: _command_result_pb2.CommandResult
    command_name: str
    command_description: str
    template_id: str
    def __init__(self, unique_id: _Optional[str] = ..., command_id: _Optional[str] = ..., execution_info: _Optional[_Union[ExecutionInfo, _Mapping]] = ..., result: _Optional[_Union[_command_result_pb2.CommandResult, _Mapping]] = ..., command_name: _Optional[str] = ..., command_description: _Optional[str] = ..., template_id: _Optional[str] = ...) -> None: ...



import ares_struct_pb2 as _ares_struct_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DeviceConnectionInfo(_message.Message):
    __slots__ = ("device_name", "device_id", "simulated", "serial_connection_info", "device_settings")
    DEVICE_NAME_FIELD_NUMBER: _ClassVar[int]
    DEVICE_ID_FIELD_NUMBER: _ClassVar[int]
    SIMULATED_FIELD_NUMBER: _ClassVar[int]
    SERIAL_CONNECTION_INFO_FIELD_NUMBER: _ClassVar[int]
    DEVICE_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    device_name: str
    device_id: str
    simulated: bool
    serial_connection_info: SerialConnection
    device_settings: _ares_struct_pb2.AresStruct
    def __init__(self, device_name: _Optional[str] = ..., device_id: _Optional[str] = ..., simulated: bool = ..., serial_connection_info: _Optional[_Union[SerialConnection, _Mapping]] = ..., device_settings: _Optional[_Union[_ares_struct_pb2.AresStruct, _Mapping]] = ...) -> None: ...

class SerialConnection(_message.Message):
    __slots__ = ("port_name", "serial_id", "baud_rate")
    PORT_NAME_FIELD_NUMBER: _ClassVar[int]
    SERIAL_ID_FIELD_NUMBER: _ClassVar[int]
    BAUD_RATE_FIELD_NUMBER: _ClassVar[int]
    port_name: str
    serial_id: str
    baud_rate: int
    def __init__(self, port_name: _Optional[str] = ..., serial_id: _Optional[str] = ..., baud_rate: _Optional[int] = ...) -> None: ...



"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(_runtime_version.Domain.PUBLIC, 6, 31, 1, '', 'templates/command_template.proto')
_sym_db = _symbol_database.Default()
from ..templates import parameter_pb2 as templates_dot_parameter__pb2
from ..templates import command_metadata_pb2 as templates_dot_command__metadata__pb2
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n templates/command_template.proto\x12\x18ares.datamodel.templates\x1a\x19templates/parameter.proto\x1a templates/command_metadata.proto"\xd3\x02\n\x0fCommandTemplate\x12\x16\n\tunique_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12;\n\x08metadata\x18\x02 \x01(\x0b2).ares.datamodel.templates.CommandMetadata\x127\n\nparameters\x18\x03 \x03(\x0b2#.ares.datamodel.templates.Parameter\x12\r\n\x05index\x18\x04 \x01(\x03\x12\\\n\x13user_output_key_map\x18\x05 \x03(\x0b2?.ares.datamodel.templates.CommandTemplate.UserOutputKeyMapEntry\x1a7\n\x15UserOutputKeyMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01B\x0c\n\n_unique_idb\x06proto3')
_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'templates.command_template_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals['_COMMANDTEMPLATE_USEROUTPUTKEYMAPENTRY']._loaded_options = None
    _globals['_COMMANDTEMPLATE_USEROUTPUTKEYMAPENTRY']._serialized_options = b'8\x01'
    _globals['_COMMANDTEMPLATE']._serialized_start = 124
    _globals['_COMMANDTEMPLATE']._serialized_end = 463
    _globals['_COMMANDTEMPLATE_USEROUTPUTKEYMAPENTRY']._serialized_start = 394
    _globals['_COMMANDTEMPLATE_USEROUTPUTKEYMAPENTRY']._serialized_end = 449


import ares_data_schema_pb2 as _ares_data_schema_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class OutputMetadata(_message.Message):
    __slots__ = ("unique_id", "data_schema", "description", "index")
    UNIQUE_ID_FIELD_NUMBER: _ClassVar[int]
    DATA_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    unique_id: str
    data_schema: _ares_data_schema_pb2.AresValueSchema
    description: str
    index: int
    def __init__(self, unique_id: _Optional[str] = ..., data_schema: _Optional[_Union[_ares_data_schema_pb2.AresValueSchema, _Mapping]] = ..., description: _Optional[str] = ..., index: _Optional[int] = ...) -> None: ...
